import * as tslib_1 from "tslib";
import "../../Core/core.scss";
import "../../Common.scss";
import "./Callout.scss";
import * as React from "react";
import { Intersection, IntersectionContext } from '../../Intersection';
import { Portal } from '../../Portal';
import { css } from '../../Util';
import { position } from '../../Utilities/Position';
var Callout = /** @class */ (function (_super) {
    tslib_1.__extends(Callout, _super);
    function Callout() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Callout.prototype.render = function () {
        var _this = this;
        return (React.createElement(Intersection, { root: document.body },
            React.createElement(IntersectionContext.Consumer, null, function (intersectionContext) { return (React.createElement(Portal, { portalElement: _this.props.hostElement, portalSelector: _this.props.hostSelector },
                React.createElement(CalloutContent, tslib_1.__assign({}, _this.props, { intersectionContext: intersectionContext })))); })));
    };
    return Callout;
}(React.Component));
export { Callout };
var CalloutContent = /** @class */ (function (_super) {
    tslib_1.__extends(CalloutContent, _super);
    function CalloutContent() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.calloutElement = React.createRef();
        _this.contentElement = React.createRef();
        // We record this before making DOM changes to prevent a forced layout
        // positioning of the callout. This does make the assumption that the page
        // wont change scroll positions while the callout is opening.
        _this.pageOffset = { x: window.pageXOffset, y: window.pageYOffset };
        _this.onIntersect = function (entries) {
            /* @TODO: Encure the callout is fully visible in the body if possible.
                    for (const entry of entries) {
                        if (entry.intersectionRatio < 1) {
                            console.log("@TODO: callout is clipped");
                            console.log(entry);
                        }
                    }
            */
        };
        _this.onKeyDown = function (event) {
            // If we press escape from within the callout this will dismiss it.
            if (_this.props.isEscDismiss && event.which === 27 /* Escape */ && !event.defaultPrevented) {
                if (_this.props.onDismiss) {
                    _this.props.onDismiss();
                }
                event.preventDefault();
            }
        };
        _this.onMouseUp = function (event) {
            // If we click on the callout not inside the content we will dismiss it.
            if (_this.props.isLightDismiss && _this.calloutElement.current === event.target && !event.defaultPrevented) {
                if (_this.props.onDismiss) {
                    _this.props.onDismiss();
                }
                event.preventDefault();
            }
        };
        return _this;
    }
    CalloutContent.prototype.render = function () {
        var _a = this.props, contentJustification = _a.contentJustification, contentLocation = _a.contentLocation, contentOrientation = _a.contentOrientation, contentSize = _a.contentSize, isLightDismiss = _a.isLightDismiss, isModal = _a.isModal;
        return (React.createElement("div", { "aria-label": this.props.ariaLabel, className: css(this.props.className, "bolt-callout absolute", contentLocation >= 0 && "absolute-fill", contentJustification === 0 /* Start */ && "justify-start", contentJustification === 1 /* Center */ && "justify-center", contentJustification === 2 /* End */ && "justify-end", contentLocation === 0 /* Start */ && "flex-start", contentLocation === 1 /* Center */ && "flex-center", contentLocation === 2 /* End */ && "flex-end", contentOrientation === 0 /* Column */ && "flex-column", contentOrientation !== 0 /* Column */ && "flex-row", isLightDismiss && "bolt-light-dismiss", isModal && "bolt-callout-modal"), id: this.props.id, onKeyDown: this.onKeyDown, onMouseUp: this.onMouseUp, ref: this.calloutElement },
            React.createElement("div", { className: css(this.props.contentClassName, "bolt-callout-content", this.props.contentShadow && "bolt-callout-shadow", contentJustification === 3 /* Stretch */ && "flex-grow", contentOrientation === 0 /* Column */ && "flex-column", contentOrientation === 1 /* Row */ && "flex-row", contentSize === 0 /* Small */ && "content-small", contentSize === 1 /* Medium */ && "content-medium", contentSize === 2 /* Large */ && "content-large"), ref: this.contentElement, role: this.props.role || "dialog" }, this.props.children)));
    };
    CalloutContent.prototype.componentDidMount = function () {
        if (this.contentElement.current) {
            this.props.intersectionContext.register(this.onIntersect);
            this.props.intersectionContext.observe(this.contentElement.current);
        }
        if (this.calloutElement.current) {
            position(this.calloutElement.current, this.props.calloutOrigin || { horizontal: "start" /* start */, vertical: "start" /* start */ }, this.props.anchorOffset, this.props.anchorElement, this.props.anchorOrigin, this.props.anchorPoint, this.pageOffset);
        }
    };
    CalloutContent.prototype.componentWillUnmount = function () {
        if (this.contentElement.current) {
            this.props.intersectionContext.unobserve(this.contentElement.current);
            this.props.intersectionContext.unregister(this.onIntersect);
        }
    };
    return CalloutContent;
}(React.Component));
