import * as tslib_1 from "tslib";
import "../../Core/core.scss";
import "../../Common.scss";
import * as React from "react";
var FocusWithin = /** @class */ (function (_super) {
    tslib_1.__extends(FocusWithin, _super);
    function FocusWithin() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.focusTimeout = -1;
        _this.focusCount = 0;
        _this.focus = false;
        /**
         * onBlur method that should be attached to the onBlur handler of the
         * continers root element.
         */
        _this.onBlur = function (event) {
            _this.focusCount--;
            if (_this.focusTimeout === -1) {
                event.persist();
                _this.focusTimeout = window.setTimeout(function () {
                    _this.focusTimeout = -1;
                    if (!_this.focusCount) {
                        _this.focus = false;
                        if (_this.props.onBlur) {
                            _this.props.onBlur(event);
                        }
                        // If we are tracking the focus state we will force a component update.
                        if (_this.props.updateStateOnFocusChange) {
                            _this.forceUpdate();
                        }
                    }
                }, 0);
            }
        };
        /**
         * onFocus method that should be attached to the onFocus handler of the
         * continer's root element.
         */
        _this.onFocus = function (event) {
            _this.focusCount++;
            // If focus is just entering one of the child components and not just moving
            // one child to another we will call the onFocus delegate if supplied.
            if (!_this.focus) {
                _this.focus = true;
                if (_this.props.onFocus) {
                    _this.props.onFocus(event);
                }
                // If we are tracking the focus state we will force a component update.
                if (_this.props.updateStateOnFocusChange) {
                    _this.forceUpdate();
                }
            }
            if (_this.focusTimeout !== -1) {
                window.clearTimeout(_this.focusTimeout);
                _this.focusTimeout = -1;
            }
        };
        return _this;
    }
    FocusWithin.prototype.render = function () {
        var newProps = {
            hasFocus: this.focus,
            onBlur: this.onBlur,
            onFocus: this.onFocus
        };
        if (typeof this.props.children === "function") {
            var child = this.props.children;
            return child(newProps);
        }
        else {
            var child = React.Children.only(this.props.children);
            return React.cloneElement(child, Object.assign({}, child.props, newProps), child.props.children);
        }
    };
    /**
     * unmount should called to prevent future events from being propagated to the
     * container and stop monitoring focus changes. This should be called if
     * components are in the process of unmounting.
     */
    FocusWithin.prototype.componentWillUnmount = function () {
        if (this.focusTimeout !== -1) {
            window.clearTimeout(this.focusTimeout);
            this.focusTimeout = -1;
        }
    };
    /**
     * hasFocus returns true if the focus is contained within the focus component
     * hierarchy. This includes portals, the element may or may not
     * be a direct descendant of the focus component in the DOM structure.
     */
    FocusWithin.prototype.hasFocus = function () {
        return this.focusCount > 0;
    };
    return FocusWithin;
}(React.Component));
export { FocusWithin };
