import "../../FabricInit";
import { IBaseProps } from "office-ui-fabric-react/lib/Utilities";
import { IContextualMenuItem } from 'office-ui-fabric-react/lib/ContextualMenu';
import { IVssIconProps } from '../../VssIcon';
import { IHeaderCommandBarItem } from '../HeaderCommandBar';
import { IObservableArray } from '../../Core/Observable';
export interface IHeader {
}

export const enum TitleSize {
    Medium,
    Large
}

export interface IHeaderProps extends IBaseProps<IHeader> {
    /**
     * An optional className to pass to the root Header element
     */
    className?: string;

    /**
     * Primary text to display in the header
     */
    title?: string;

    /**
     * Title Font Size, informs:
     * Title Font Size
     * Command Bar button height
     * Icon height
     */
    titleSize?: TitleSize;

    /**
     * Small, secondary text to render above the title text
     */
    kicker?: string;

    /**
     * Custom rendering logic for the Title element
     * Will ignore title and titleSize font size if provided
     * titleSize should still be provided to inform Command and View Bar button height
     * as well as icon height
     */
    onRenderTitle?: () => JSX.Element;

    /**
     * Custom rendering logic for content under the overall Header
     */
    onRenderCustomAreaContent?: () => JSX.Element;

    /**
     * Props to render the large Title Icon to the left of the Title and Kicker elements
     */
    titleIconProps?: IVssIconProps;

    /**
     * Items to display in the far-right Command Bar
     * Should influence page-wide state
     */
    commandBarItems?: IObservableArray<IHeaderCommandBarItem> | IHeaderCommandBarItem[];

    /**
     * Items to display in the View Actions Bar, to the left of the Command Bar (if it exists)
     * Do not use if your page content includes a PivotBar; include your View Actions on the PivotBar
     */
    viewItems?: IObservableArray<IHeaderCommandBarItem> | IHeaderCommandBarItem[];
}