import * as tslib_1 from "tslib";
import "../../Core/core.scss";
import "../../Common.scss";
import * as React from "react";
// We need to monitor fine grained changes, especially when the list
// has horizontal scroll. You dont get 100% visible ever.
var threshold = [];
for (var index = 0; index <= 100; index++) {
    threshold.push(index / 100);
}
var IntersectionContextImpl = /** @class */ (function () {
    function IntersectionContextImpl() {
        var _this = this;
        this.scrollTop = 0;
        this.callbacks = [];
        this.pending = [];
        this.onIntersect = function (entries) {
            if (_this.observer) {
                for (var _i = 0, _a = _this.callbacks; _i < _a.length; _i++) {
                    var callback = _a[_i];
                    callback(entries, _this.observer);
                }
            }
        };
    }
    IntersectionContextImpl.prototype.connect = function (root, rootMargin) {
        this.observer = new IntersectionObserver(this.onIntersect, { root: root, rootMargin: rootMargin, threshold: threshold });
        for (var _i = 0, _a = this.pending; _i < _a.length; _i++) {
            var element = _a[_i];
            this.observer.observe(element);
        }
    };
    IntersectionContextImpl.prototype.disconnect = function () {
        if (this.observer) {
            this.observer.disconnect();
        }
    };
    IntersectionContextImpl.prototype.getScrollTop = function () {
        var scrollElement = document.body;
        if (this.observer && this.observer.root) {
            scrollElement = this.observer.root;
        }
        return scrollElement.scrollTop;
    };
    IntersectionContextImpl.prototype.observe = function (element) {
        if (this.observer) {
            this.observer.observe(element);
        }
        else {
            this.pending.push(element);
        }
    };
    IntersectionContextImpl.prototype.register = function (callback) {
        this.callbacks.push(callback);
    };
    IntersectionContextImpl.prototype.unobserve = function (element) {
        var elementIndex = this.pending.indexOf(element);
        if (elementIndex >= 0) {
            this.pending.splice(elementIndex, 1);
        }
        if (this.observer) {
            this.observer.unobserve(element);
        }
    };
    IntersectionContextImpl.prototype.unregister = function (callback) {
        var callbackIndex = this.callbacks.indexOf(callback);
        if (callbackIndex >= 0) {
            this.callbacks.splice(callbackIndex, 1);
        }
    };
    return IntersectionContextImpl;
}());
export var IntersectionContext = React.createContext(new IntersectionContextImpl());
var Intersection = /** @class */ (function (_super) {
    tslib_1.__extends(Intersection, _super);
    function Intersection(props) {
        var _this = _super.call(this, props) || this;
        _this.state = new IntersectionContextImpl();
        // If a targetted root was supplied we will initialize with this element.
        _this.rootElement = props.root;
        return _this;
    }
    // Render the provider around a SINGLE child. This is the element that is scrollable.
    Intersection.prototype.render = function () {
        var _this = this;
        if (!this.props.root) {
            var child = React.Children.only(this.props.children);
            return (React.createElement(IntersectionContext.Provider, { value: this.state }, React.cloneElement(child, tslib_1.__assign({}, Object.assign({}, child.props, { ref: function (element) { return (_this.rootElement = element); } })))));
        }
        else {
            return React.createElement(IntersectionContext.Provider, { value: this.state }, this.props.children);
        }
    };
    Intersection.prototype.componentDidMount = function () {
        if (this.rootElement) {
            this.state.connect(this.rootElement, this.props.rootMargin);
            // Allow the creator of the intersection to observe intersection events.
            if (this.props.onIntersect) {
                this.state.register(this.props.onIntersect);
            }
        }
    };
    Intersection.prototype.componentWillUnmount = function () {
        this.state.disconnect();
    };
    return Intersection;
}(React.Component));
export { Intersection };
