import * as React from "react";
import { IFocusZoneProps } from '../../FocusZone';
import { IIconProps } from '../../Icon';
import { IIntersectionContext } from '../../Intersection';
import { IBehavior, IEventDispatch } from '../../Utilities/Behavior';
import { IIndexed, ISelection, ISelectableUI } from '../../Utilities/Selection';
import { IItemProvider } from '../../Utilities/Provider';
export interface IListRow<T> extends IIndexed {}

export interface IListSelection extends ISelection, IBehavior<any, ISelectableUI> {}

export interface IList<T> extends ISelectableUI {
    getFocusIndex(): number;
    indexFromPoint(clientX: number, clientY: number): number;
}

export interface IListProps<T> {
    itemProvider: IItemProvider<T>;

    behaviors?: IBehavior<Partial<IListProps<T>>, Partial<IList<T>>>[];
    className?: string;
    columns?: number;
    focuszoneProps?: IFocusZoneProps;
    eventDispatch?: IEventDispatch;
    id?: string;
    initialPageCount?: number;
    maxWidth?: string;
    minWidth?: string;
    pageSize?: number;
    role?: string;
    rowHeight?: number;
    selection?: IListSelection;
    width?: string;

    onRenderRow: (
        index: number,
        item: T,
        position: number,
        setSize: number,
        onFocusItem: (rowIndex: number, event: React.FocusEvent<HTMLElement>) => void,
        props: IListProps<T>,
        eventDispatch: IEventDispatch
    ) => JSX.Element;

    onRenderHeader?: () => JSX.Element;

    onActivate?: (event: React.SyntheticEvent<HTMLElement>, listRow: IListRow<T>) => void;
    onSelect?: (event: React.SyntheticEvent<HTMLElement>, listRow: IListRow<T>) => void;
}

export interface ISimpleListCell {
    iconProps?: IIconProps;
    text?: string;
    textClassName?: string;
    href?: string;
}

export interface ISimpleListProps {
    itemProvider: IItemProvider<ISimpleListCell | string | number>;

    behaviors?: Array<IBehavior<Partial<IListProps<ISimpleListCell | string | number>>, Partial<IList<ISimpleListCell | string | number>>>>;
    className?: string;
    eventDispatch?: IEventDispatch;
    focuszoneProps?: IFocusZoneProps;
    id?: string;
    rowHeight?: number;
    selection?: IListSelection;
    scrollable?: true;
    width?: string;

    onActivate?: (event: React.SyntheticEvent<HTMLElement>, listRow: IListRow<ISimpleListCell | string | number>) => void;
    onSelect?: (event: React.SyntheticEvent<HTMLElement>, listRow: IListRow<ISimpleListCell | string | number>) => void;
}