import * as tslib_1 from "tslib";
import "../../Core/core.scss";
import "../../Common.scss";
import "./List.scss";
import * as React from "react";
import { FocusZone, FocusZoneContext } from '../../FocusZone';
import { Icon } from '../../Icon';
import { Intersection, IntersectionContext } from '../../Intersection';
import { Tooltip } from '../../TooltipEx';
import { css } from '../../Util';
import { EventDispatch } from '../../Utilities/Dispatch';
var List = /** @class */ (function (_super) {
    tslib_1.__extends(List, _super);
    function List(props) {
        var _this = _super.call(this, props) || this;
        // Track the table element used to render the rows.
        _this.listElement = React.createRef();
        // Manage data about pages, including their spacers.
        _this.spacerElements = {};
        _this.pageDetails = {};
        // Track the pages that are rendered as focus pages.
        _this.firstFocusPage = -1;
        _this.lastFocusPage = -1;
        _this.onDispatch = function (event) {
            _this.state.eventDispatch.dispatchEvent(event);
        };
        _this.onDoubleClick = function (event) {
            _this.onDispatch(event);
            if (!event.defaultPrevented) {
                var rowIndex = _this.indexFromPoint(event.clientX, event.clientY);
                if (rowIndex >= 0) {
                    _this.onActivate(event, { index: rowIndex });
                }
            }
        };
        _this.onFocusItem = function (rowIndex, event) {
            if (_this.state.focusIndex !== rowIndex) {
                // We are going to pass the event along after state is updated, we need to ensure
                // the event doesnt get re-used.
                event.persist();
                _this.setState({ focusIndex: rowIndex }, function () {
                    // We need to re-render the row so we will clear the cached value.
                    _this.state.renderedRows[rowIndex] = null;
                    _this.onSelect(event, { index: rowIndex });
                });
            }
        };
        _this.onKeyDown = function (event) {
            _this.onDispatch(event);
            if (!event.defaultPrevented) {
                var focusIndex = _this.state.focusIndex;
                if (event.which === 13 /* Enter */) {
                    if (focusIndex >= 0) {
                        _this.onActivate(event, { index: focusIndex });
                    }
                    event.preventDefault();
                }
                else if (event.which === 32 /* Space */) {
                    event.preventDefault();
                }
            }
        };
        _this.onIntersect = function (entries) {
            var pagesChanged = {};
            var scrollTop = _this.intersectionContext.getScrollTop(); // @TODO: - this.listElement.current.offsetTop;
            var recomputeViewport = false;
            var paddingIntersection = false;
            var pageIntersection = false;
            var rowHeight;
            // First process all instersection entries for sentinel visibility changes.
            for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
                var entry = entries_1[_i];
                // First determine which page was affected, we can scan the currently
                // loaded pageElements to find the correct one.
                for (var pageIndexKey in _this.pageDetails) {
                    var pageDetail = _this.pageDetails[pageIndexKey];
                    if (entry.target === pageDetail.pageElement) {
                        // If a rowHeight was specified we will compute on based on the average rowHeight in the
                        // first page rendered.
                        if (_this.state.rowHeight === 0) {
                            // @NOTE: Due to a bug in Chrome where the clientBoundingRect is not correct we will
                            //  have to get the actual value. This shouldnt cause a layout at this point.
                            rowHeight = entry.target.getBoundingClientRect().height / Math.min(_this.state.rowCount, _this.state.pageSize);
                        }
                        pageDetail.topLocation =
                            entry.boundingClientRect.top < entry.rootBounds.top
                                ? -1 /* Above */
                                : entry.boundingClientRect.top > entry.rootBounds.bottom
                                    ? 1 /* Below */
                                    : 0 /* Visible */;
                        pageDetail.bottomLocation =
                            entry.boundingClientRect.bottom < entry.rootBounds.top
                                ? -1 /* Above */
                                : entry.boundingClientRect.bottom > entry.rootBounds.bottom
                                    ? 1 /* Below */
                                    : 0 /* Visible */;
                        // Mark this page as changed, this will help determine the viewport changes required.
                        pagesChanged[pageDetail.pageIndex] = pageDetail.pageIndex;
                        pageIntersection = true;
                        break;
                    }
                }
                // If the intersection change is on one of the spacer elements check if it is visible.
                for (var key in _this.spacerElements) {
                    var spacerElement = _this.spacerElements[key];
                    if (entry.target === spacerElement) {
                        if (entry.boundingClientRect.top <= entry.rootBounds.bottom && entry.rootBounds.top <= entry.boundingClientRect.bottom) {
                            paddingIntersection = true;
                        }
                    }
                }
            }
            // Now that we have processed all pending observer entries, we need to
            // update the state of the list.
            var maxPage = _this.state.maxPage;
            var _a = _this.state, firstPage = _a.firstPage, lastPage = _a.lastPage;
            for (var key in pagesChanged) {
                var currentPageDetail = _this.pageDetails[key];
                var bottomLocation = currentPageDetail.bottomLocation;
                var topLocation = currentPageDetail.topLocation;
                var pageIndex = currentPageDetail.pageIndex;
                // Determine the location of both the top and bottom sentinel.
                var bottomVisible = bottomLocation === 0 /* Visible */;
                var topVisible = topLocation === 0 /* Visible */;
                if (!topVisible && topLocation === bottomLocation) {
                    if (topLocation === -1 /* Above */) {
                        firstPage = Math.max(firstPage, Math.min(pageIndex + 1, maxPage));
                        if (firstPage > _this.state.lastPage) {
                            recomputeViewport = true;
                        }
                    }
                    else {
                        lastPage = Math.min(lastPage, Math.max(0, pageIndex - 1));
                        if (lastPage < _this.state.firstPage) {
                            recomputeViewport = true;
                        }
                    }
                }
                else {
                    if (topVisible) {
                        firstPage = Math.min(firstPage, Math.max(0, pageIndex - 1));
                    }
                    if (bottomVisible) {
                        lastPage = Math.max(lastPage, Math.min(pageIndex + 1, maxPage));
                    }
                }
            }
            // If we have scrolled all pages out of view, we will compute the top page
            // based on the scrollTop of the intersectionContext.
            if (recomputeViewport || (paddingIntersection && !pageIntersection)) {
                firstPage = Math.max(0, Math.min(_this.state.maxPage, Math.floor(scrollTop / _this.state.rowHeight / _this.state.pageSize)));
                lastPage = firstPage + (_this.props.initialPageCount - 1);
            }
            if (firstPage !== _this.state.firstPage || lastPage !== _this.state.lastPage || rowHeight) {
                _this.setState({ firstPage: firstPage, lastPage: lastPage, rowHeight: rowHeight || _this.state.rowHeight });
            }
        };
        var rowCount = props.itemProvider.getTotal();
        var pageSize = props.pageSize || 10;
        var maxPage = rowCount === -1 ? Number.MAX_SAFE_INTEGER : Math.ceil(rowCount / pageSize) - 1;
        _this.state = {
            eventDispatch: props.eventDispatch || new EventDispatch(),
            firstPage: 0,
            focusIndex: -1,
            itemProvider: props.itemProvider,
            lastPage: Math.min(_this.props.initialPageCount - 1, maxPage),
            maxPage: maxPage,
            pageSize: pageSize,
            providerVersion: props.itemProvider.getVersion(),
            renderedRows: [],
            rowCount: rowCount,
            rowHeight: props.rowHeight || 0,
            rows: [],
            scrollTop: 0
        };
        // The selection is implemented as a behavior, we will initialize it.
        if (props.selection) {
            props.selection.initialize(props, _this, _this.state.eventDispatch);
        }
        // Initialize the supplied behaviors.
        if (props.behaviors) {
            for (var _i = 0, _a = props.behaviors; _i < _a.length; _i++) {
                var behavior = _a[_i];
                behavior.initialize(props, _this, _this.state.eventDispatch);
            }
        }
        return _this;
    }
    List.getDerivedStateFromProps = function (props, state) {
        // If the provider has changed we need to clear any cached data about the rows.
        if (props.itemProvider !== state.itemProvider || state.providerVersion !== props.itemProvider.getVersion()) {
            return {
                itemProvider: props.itemProvider,
                providerVersion: props.itemProvider.getVersion(),
                renderedRows: [],
                rowCount: props.itemProvider.getTotal(),
                rows: []
            };
        }
        return null;
    };
    List.prototype.render = function () {
        var _this = this;
        var _a = this.props, className = _a.className, focuszoneProps = _a.focuszoneProps, id = _a.id, maxWidth = _a.maxWidth, minWidth = _a.minWidth, role = _a.role, width = _a.width;
        var _b = this.state, focusIndex = _b.focusIndex, maxPage = _b.maxPage, pageSize = _b.pageSize;
        var rows = [];
        // We want to render overflow pages on either end of the visible range to help smooth out scrolling.
        var firstPage = Math.max(0, this.state.firstPage - 0);
        var lastPage = Math.min(maxPage, this.state.lastPage + 0);
        // Number of pages each spacer takes up. There are potentially two spacers above
        // or below the view port. They surround the focus range when the focus range is
        // not within the viewport.
        var topSpacer1 = 0;
        var topSpacer2 = firstPage;
        var bottomSpacer2 = maxPage - lastPage;
        var bottomSpacer1 = 0;
        // Compute the range of focus pages, these will be either before or after the pages
        // in the viewport. We need to ensure we have one row before and one row after the
        // focus row to support arrowing up and down.
        if (focusIndex !== -1) {
            this.firstFocusPage = Math.max(0, Math.floor((focusIndex - 3) / pageSize));
            this.lastFocusPage = Math.min(maxPage, Math.floor((focusIndex + 3) / pageSize));
            // Make sure we dont draw any of the pages that are in the viewport.
            if (this.firstFocusPage < firstPage) {
                this.lastFocusPage = Math.min(this.lastFocusPage, firstPage - 1);
                topSpacer1 = this.firstFocusPage;
                topSpacer2 = firstPage - this.lastFocusPage - 1;
            }
            else if (this.lastFocusPage > lastPage) {
                this.firstFocusPage = Math.max(this.firstFocusPage, lastPage + 1);
                bottomSpacer2 = this.firstFocusPage - lastPage - 1;
                bottomSpacer1 = maxPage - this.lastFocusPage;
            }
            else {
                this.firstFocusPage = -1;
                this.lastFocusPage = -1;
            }
        }
        this.renderSpacer("st1", topSpacer1, rows);
        // If the focus pages are before the viewport render them up too
        // the first page but not including the first page.
        if (this.firstFocusPage < firstPage && this.firstFocusPage !== -1) {
            for (var pageIndex = this.firstFocusPage; pageIndex <= this.lastFocusPage; pageIndex++) {
                rows.push(this.renderPage(pageIndex));
            }
        }
        this.renderSpacer("st2", topSpacer2, rows);
        // Go through each of the rendered pages and generate the child component.
        for (var pageIndex = firstPage; pageIndex <= lastPage; pageIndex++) {
            rows.push(this.renderPage(pageIndex));
        }
        this.renderSpacer("sb2", bottomSpacer2, rows);
        // If the focus pages are after the last page in the viewport render
        // them but not including the last page.
        if (this.lastFocusPage > lastPage) {
            for (var pageIndex = this.firstFocusPage; pageIndex <= this.lastFocusPage; pageIndex++) {
                rows.push(this.renderPage(pageIndex));
            }
        }
        this.renderSpacer("sb1", bottomSpacer1, rows);
        return (React.createElement(IntersectionContext.Consumer, null, function (intersectionContext) {
            _this.intersectionContext = intersectionContext;
            var list = (React.createElement("table", { className: css(className, "bolt-list"), id: id, onClick: _this.onDispatch, onDoubleClick: _this.onDoubleClick, onKeyDown: _this.onKeyDown, onKeyUp: _this.onDispatch, onMouseDown: _this.onDispatch, role: role || "listbox", style: { maxWidth: maxWidth, minWidth: minWidth, width: width } },
                _this.props.onRenderHeader && _this.props.onRenderHeader(),
                rows));
            if (focuszoneProps) {
                list = React.createElement(FocusZone, tslib_1.__assign({}, focuszoneProps), list);
            }
            return list;
        }));
    };
    List.prototype.componentDidMount = function () {
        if (this.intersectionContext) {
            this.intersectionContext.register(this.onIntersect);
        }
    };
    List.prototype.componentDidUpdate = function () {
        if (this.intersectionContext) {
            for (var pageIndexKey in this.pageDetails) {
                var pageIndex = parseInt(pageIndexKey, 10);
                if ((pageIndex < this.state.firstPage || pageIndex > this.state.lastPage) &&
                    (pageIndex < this.firstFocusPage || pageIndex > this.lastFocusPage)) {
                    var pageDetail = this.pageDetails[pageIndex];
                    if (pageDetail) {
                        if (pageDetail.pageElement) {
                            this.intersectionContext.unobserve(pageDetail.pageElement);
                        }
                        delete this.pageDetails[pageIndex];
                    }
                }
            }
        }
    };
    List.prototype.componentWillUnmount = function () {
        if (this.intersectionContext) {
            this.intersectionContext.unregister(this.onIntersect);
        }
    };
    List.prototype.getFocusIndex = function () {
        return this.state.focusIndex;
    };
    List.prototype.itemFromPoint = function (clientX, clientY) {
        var rowIndex = this.indexFromPoint(clientX, clientY);
        if (rowIndex >= 0) {
            return this.props.itemProvider.getItem(rowIndex);
        }
        return undefined;
    };
    List.prototype.indexFromPoint = function (clientX, clientY) {
        var attributeValue;
        var rowIndex = -1;
        var element = document.elementFromPoint(clientX, clientY);
        while (element) {
            attributeValue = element.getAttribute("data-row-index");
            if (attributeValue) {
                rowIndex = parseInt(attributeValue, 10);
                break;
            }
            // We have hit the root of the details list, dont look above this.
            if (element.classList.contains("list")) {
                break;
            }
            element = element.parentElement;
        }
        return rowIndex;
    };
    List.prototype.onActivate = function (event, listRow) {
        this.state.eventDispatch.dispatchEvent(event, listRow, "activate");
        if (this.props.onActivate) {
            this.props.onActivate(event, listRow);
        }
    };
    List.prototype.onSelect = function (event, listRow) {
        this.state.eventDispatch.dispatchEvent(event, listRow, "select");
        if (this.props.onSelect) {
            this.props.onSelect(event, listRow);
        }
    };
    List.prototype.renderPage = function (pageIndex) {
        var _this = this;
        var _a = this.props, itemProvider = _a.itemProvider, onRenderRow = _a.onRenderRow;
        var _b = this.state, pageSize = _b.pageSize, renderedRows = _b.renderedRows, rows = _b.rows;
        var firstIndex = pageIndex * pageSize;
        var lastIndex = firstIndex + pageSize;
        var pageRows = [];
        var _loop_1 = function (index) {
            var renderedRow = renderedRows[index];
            if (!renderedRow) {
                if (!rows[index]) {
                    var possibleItem = itemProvider.getItem(index);
                    // If there is no row, we are done with the rows from the provider.
                    if (!possibleItem) {
                        return "break";
                    }
                    // Detemine if the item is a promise or an actual item.
                    if (possibleItem.then) {
                        possibleItem.then(function (item) {
                            if (item) {
                                rows[index] = item;
                                _this.forceUpdate();
                            }
                        });
                        return "break";
                    }
                    else {
                        rows[index] = possibleItem;
                    }
                }
                renderedRow = renderedRows[index] = onRenderRow(index, rows[index], itemProvider.getPosition(index), itemProvider.getCount(), this_1.onFocusItem, this_1.props, this_1.state.eventDispatch);
            }
            pageRows.push(renderedRow);
        };
        var this_1 = this;
        // Render all the rows we have in the viewpport.
        for (var index = firstIndex; index < lastIndex; index++) {
            var state_1 = _loop_1(index);
            if (state_1 === "break")
                break;
        }
        return (React.createElement("tbody", { key: pageIndex, ref: function (pageElement) { return _this.setPageElement(pageIndex, pageElement); }, "data-page": pageIndex }, pageRows));
    };
    List.prototype.renderSpacer = function (key, pages, rows) {
        var _this = this;
        if (this.intersectionContext) {
            rows.push(React.createElement("tbody", { key: key, ref: function (spacerElement) { return _this.setSpacerElement(key, spacerElement); } },
                React.createElement("tr", { className: "bolt-list-row-spacer" },
                    React.createElement("td", { className: "bolt-list-cell-spacer", colSpan: this.props.columns, style: { height: pages * this.state.rowHeight * this.state.pageSize + "px" } }))));
        }
    };
    List.prototype.setPageElement = function (pageIndex, pageElement) {
        if (pageElement) {
            if (this.intersectionContext) {
                var pageDetail = this.pageDetails[pageIndex];
                // If we already have sentinel information for this page, we will unobserve, the pageElement
                if (pageDetail) {
                    // If the pageElement hasnt changed, we are done.
                    if (pageDetail.pageElement === pageElement) {
                        return;
                    }
                    if (pageDetail.pageElement) {
                        this.intersectionContext.unobserve(pageDetail.pageElement);
                    }
                    // Update the page element for this page
                    pageDetail.pageElement = pageElement;
                }
                else {
                    pageDetail = this.pageDetails[pageIndex] = {
                        bottomLocation: 0 /* Visible */,
                        pageElement: pageElement,
                        pageIndex: pageIndex,
                        topLocation: 0 /* Visible */
                    };
                }
                // Observe the elements that surround our page
                this.intersectionContext.observe(pageElement);
            }
        }
    };
    List.prototype.setSpacerElement = function (key, spacerElement) {
        if (spacerElement) {
            if (this.intersectionContext) {
                var existingElement = this.spacerElements[key];
                if (existingElement !== spacerElement) {
                    if (existingElement) {
                        this.intersectionContext.unobserve(spacerElement);
                    }
                    this.intersectionContext.observe(spacerElement);
                }
                this.spacerElements[key] = spacerElement;
            }
        }
    };
    List.defaultProps = {
        columns: 1,
        focuszoneProps: { direction: 2 /* Vertical */ },
        initialPageCount: 2,
        role: "listbox"
    };
    return List;
}(React.Component));
export { List };
var ScrollableList = /** @class */ (function (_super) {
    tslib_1.__extends(ScrollableList, _super);
    function ScrollableList() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.list = React.createRef();
        return _this;
    }
    ScrollableList.prototype.render = function () {
        return (React.createElement(Intersection, { rootMargin: window.innerHeight / 2 + "px" },
            React.createElement("div", { className: "flex-grow scroll-auto" },
                React.createElement(List, tslib_1.__assign({}, this.props, { ref: this.list })))));
    };
    ScrollableList.prototype.getFocusIndex = function () {
        return this.list.current.getFocusIndex();
    };
    ScrollableList.prototype.indexFromPoint = function (clientX, clientY) {
        return this.list.current.indexFromPoint(clientX, clientY);
    };
    return ScrollableList;
}(React.Component));
export { ScrollableList };
var SimpleList = /** @class */ (function (_super) {
    tslib_1.__extends(SimpleList, _super);
    function SimpleList() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onRenderListItem = function (index, listCell, position, setSize, onFocusItem, props) {
            return (React.createElement(ListItem, { key: index, index: index, listCell: listCell, listProps: props, onFocusItem: onFocusItem, position: position, setSize: setSize }));
        };
        return _this;
    }
    SimpleList.prototype.render = function () {
        var listProps = {
            className: css(this.props.className, "list"),
            columns: 1,
            eventDispatch: this.props.eventDispatch,
            focuszoneProps: this.props.focuszoneProps,
            id: this.props.id,
            itemProvider: this.props.itemProvider,
            onActivate: this.props.onActivate,
            onRenderRow: this.onRenderListItem,
            onSelect: this.props.onSelect,
            rowHeight: this.props.rowHeight,
            selection: this.props.selection,
            width: this.props.width
        };
        if (this.props.scrollable) {
            return React.createElement(ScrollableList, tslib_1.__assign({}, listProps));
        }
        else {
            return React.createElement(List, tslib_1.__assign({}, listProps));
        }
    };
    return SimpleList;
}(React.Component));
export { SimpleList };
var ListItem = /** @class */ (function (_super) {
    tslib_1.__extends(ListItem, _super);
    function ListItem() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onFocus = function (event) {
            _this.props.onFocusItem(_this.props.index, event);
        };
        _this.onSelectionChange = function (selectedRanges) {
            var index = _this.props.index;
            for (var _i = 0, selectedRanges_1 = selectedRanges; _i < selectedRanges_1.length; _i++) {
                var selectionRange = selectedRanges_1[_i];
                if (index >= selectionRange.beginIndex && index <= selectionRange.endIndex) {
                    _this.forceUpdate();
                }
            }
        };
        return _this;
    }
    ListItem.prototype.render = function () {
        var _this = this;
        var _a = this.props, index = _a.index, listCell = _a.listCell, listProps = _a.listProps, position = _a.position, setSize = _a.setSize;
        var selection = listProps.selection;
        var selected = selection && selection.selected(index);
        return (React.createElement(FocusZoneContext.Consumer, null, function (rowContext) {
            return (React.createElement(FocusZone, { direction: 1 /* Horizontal */ },
                React.createElement("tr", { "aria-posinset": position, "aria-setsize": setSize, className: css("bolt-list-row", selected && "selected"), "data-focuszone": rowContext.focuszoneId, "data-row-index": index, tabIndex: -1, onFocus: _this.onFocus },
                    React.createElement("td", { className: "bolt-list-cell" },
                        React.createElement("div", { className: "bolt-list-cell-content flex-row" }, renderListCell(listCell))))));
        }));
    };
    ListItem.prototype.componentDidMount = function () {
        var selection = this.props.listProps.selection;
        if (selection) {
            selection.subscribe(this.onSelectionChange);
        }
    };
    ListItem.prototype.componentWillUnmount = function () {
        var selection = this.props.listProps.selection;
        if (selection) {
            selection.unsubscribe(this.onSelectionChange);
        }
    };
    return ListItem;
}(React.Component));
export function renderListCell(listCell) {
    return typeof listCell === "string" || typeof listCell === "number" ? (React.createElement("span", { className: "bolt-list-cell-text bolt-list-cell-child flex-row" },
        React.createElement(Tooltip, { overflowOnly: true },
            React.createElement("span", { className: "text-ellipsis" }, listCell)))) : (React.createElement(React.Fragment, null,
        listCell.iconProps && Icon(listCell.iconProps),
        listCell.text && (React.createElement("span", { className: css(listCell.textClassName, "bolt-list-cell-text bolt-list-cell-child flex-row", listCell.href && "bolt-list-cell-link") },
            React.createElement(Tooltip, { overflowOnly: true }, listCell.href ? (React.createElement(FocusZoneContext.Consumer, null, function (cellContext) { return (React.createElement("a", { className: "text-ellipsis", href: listCell.href, "data-focuszone": cellContext.focuszoneId, tabIndex: -1 }, listCell.text)); })) : (React.createElement("span", { className: "text-ellipsis" }, listCell.text)))))));
}
