import * as React from "react";
import { IReadonlyObservableValue } from '../../Core/Observable';
import { IIconProps } from '../../Icon';
import { IOffset, IOrigin, IPoint } from '../../Utilities/Position';
/**
 * IMenu interface allows callers to interact with the menu through
 * an API outside of the props.
 */
export interface IMenu {
    expandItem: (menuItem: IMenuItem | null, expanded: boolean) => void;
    onActivateItem: (menuItem: IMenuItem, event?: React.MouseEvent<HTMLElement> | React.KeyboardEvent<HTMLElement>) => void;
}

/**
 * Standard menu item types that are rendered through the menu.
 */
export const enum MenuItemType {
    Normal = 0,
    Divider = 1,
    Header = 2
}

/**
 * IMenuItem represents a single menu item in a ContextualMenu.
 */
export interface IMenuItem {
    className?: string;
    checked?: IReadonlyObservableValue<boolean> | boolean;
    disabled?: boolean;
    href?: string;
    iconProps?: IIconProps;
    id: string;
    itemType?: MenuItemType;
    readonly?: boolean;
    secondaryText?: string;
    subMenuProps?: IMenuProps;
    text?: string;

    onActivateItem?: (menuItem: IMenuItem, event?: React.MouseEvent<HTMLElement> | React.KeyboardEvent<HTMLElement>) => boolean | void;

    onRenderMenuIcon?: (menuItem: IMenuItem) => JSX.Element;
    onRenderMenuItem?: (
        index: number,
        item: IMenuItem,
        position: number,
        setSize: number,
        expandedIndex: IObservableValue<number>,
        expandItem: (menuItem: IMenuItem, expanded: boolean) => void,
        onActivateItem: (menuItem: IMenuItem, event?: React.MouseEvent<HTMLElement> | React.KeyboardEvent<HTMLElement>) => void,
        onFocusItem: (index: number, event: React.FocusEvent<HTMLElement>) => void
    ) => JSX.Element;
}

export interface IMenuProps {
    className?: string;
    id: string;
    items?: IMenuItem[];

    onActivateItem?: (menuItem: IMenuItem, event?: React.MouseEvent<HTMLElement> | React.KeyboardEvent<HTMLElement>) => void;
}

export interface IContextualMenu extends IMenu {
    dismiss: (dismissAll?: boolean) => void;
}

export interface IContextualMenuProps {
    anchorElement?: HTMLElement;
    anchorOrigin?: IOrigin;
    anchorOffset?: IOffset;
    anchorPoint?: IPoint;
    closeOnFocus?: boolean;
    hostSelector?: string;
    isSubMenu?: boolean;
    menuOrigin?: IOrigin;
    menuProps: IMenuProps;

    onDismiss?: (dismissAll?: boolean) => void;
}