import "../../Core/core.scss";
import "../../Common.scss";
import "./Menu.scss";
import "./MenuButton.scss";
import * as React from "react";
import { IReadonlyObservableValue, ObservableValue } from '../../Core/Observable';
import { ArrayItemProvider } from '../../Utilities/Provider';
import { IContextualMenuProps, IMenu, IMenuItem, IMenuProps } from "./Menu.Props";
/**
 * Known issues:
 *  Firefox - Bug 1424328 - Scrollbar overlaps content on menus that have vertical overflow (div with overflow-y auto).
 */
export interface IMenuState {
    expandedIndex: ObservableValue<number>;
    itemProvider: MenuItemProvider;
    items: IMenuItem[];
}
export declare class Menu extends React.Component<IMenuProps, IMenuState> implements IMenu {
    static getDerivedStateFromProps(props: Readonly<IMenuProps>, state: Readonly<IMenuState>): Partial<IMenuState> | null;
    static isDecorative(menuItem: IMenuItem): boolean;
    private static focuszoneProps;
    constructor(props: Readonly<IMenuProps>);
    render(): JSX.Element;
    expandItem: (menuItem: IMenuItem, expanded: boolean) => void;
    onActivateItem: (menuItem: IMenuItem, event?: React.KeyboardEvent<HTMLElement> | React.MouseEvent<HTMLElement>) => void;
    private renderMenuItem;
}
export declare function MenuDivider(menuItem: IMenuItem, index: number): JSX.Element;
export declare function MenuHeader(menuItem: IMenuItem, index: number): JSX.Element;
declare class MenuItemProvider extends ArrayItemProvider<IMenuItem> {
    private positions;
    private count;
    getCount(): number;
    getPosition(index: number): number;
}
export interface IMenuItemProps {
    expandedIndex: IReadonlyObservableValue<number>;
    index: number;
    menuItem: IMenuItem;
    position: number;
    setSize: number;
    expandItem: (menuItem: IMenuItem, expanded: boolean) => void;
    onActivateItem: (menuItem: IMenuItem, event?: React.MouseEvent<HTMLElement> | React.KeyboardEvent<HTMLElement>) => void;
    onFocusItem: (index: number, event: React.FocusEvent<HTMLElement>) => void;
}
export interface IMenuItemState {
    expanded: boolean;
}
export declare class MenuItem extends React.Component<IMenuItemProps, IMenuItemState> {
    private localKeyStroke;
    private closeTimeout;
    private openTimeout;
    private element;
    constructor(props: IMenuItemProps);
    render(): JSX.Element;
    componentDidMount(): void;
    componentWillUnmount(): void;
    private handleClick;
    private onClick;
    private onDismiss;
    private onExpandedChange;
    private onFocus;
    private onKeyDown;
    private onKeyUp;
    private onMouseDown;
    private onMouseEnter;
    private onMouseLeave;
}
export interface IContextualMenuState {
    expandedIndex?: number;
    items: IMenuItem[];
    overflow: boolean;
}
export declare class ContextualMenu extends React.Component<IContextualMenuProps, IContextualMenuState> {
    static defaultProps: Readonly<Partial<IContextualMenuProps>>;
    private previousFocusElement?;
    private focusWithin;
    private menu;
    constructor(props: Readonly<IContextualMenuProps>);
    render(): JSX.Element;
    componentDidMount(): void;
    componentWillUnmount(): void;
    dismiss: (dismissAll?: boolean) => void;
    private onActivateItem;
    private onBlur;
    private onIntersect;
    private onKeyDown;
    private onScroll;
}
export {};
