import * as tslib_1 from "tslib";
import "../../Core/core.scss";
import "../../Common.scss";
import "./Menu.scss";
import "./MenuButton.scss";
import * as React from "react";
import { ObservableLike, ObservableValue } from '../../Core/Observable';
import { Callout } from '../../callout';
import { Checkbox } from '../../checkbox';
import { FocusWithin } from '../../FocusWithin';
import { FocusZoneContext } from '../../FocusZone';
import { Icon } from '../../Icon';
import { Intersection } from '../../Intersection';
import { List } from '../../List';
import { ArrayItemProvider } from '../../Utilities/Provider';
import { css, preventDefault } from '../../Util';
var Menu = /** @class */ (function (_super) {
    tslib_1.__extends(Menu, _super);
    function Menu(props) {
        var _this = _super.call(this, props) || this;
        _this.expandItem = function (menuItem, expanded) {
            var items = _this.state.items;
            if (!menuItem && _this.state.expandedIndex.value !== -1) {
                menuItem = items[_this.state.expandedIndex.value];
            }
            if (menuItem && menuItem.subMenuProps) {
                var index = items.indexOf(menuItem);
                if (index >= 0) {
                    if (expanded) {
                        _this.state.expandedIndex.value = index;
                    }
                    else {
                        _this.state.expandedIndex.value = -1;
                    }
                }
            }
        };
        _this.onActivateItem = function (menuItem, event) {
            if (_this.props.onActivateItem) {
                _this.props.onActivateItem(menuItem, event);
            }
        };
        _this.renderMenuItem = function (index, menuItem, position, setSize, onFocusItem) {
            if (menuItem.onRenderMenuItem) {
                return menuItem.onRenderMenuItem(index, menuItem, position, setSize, _this.state.expandedIndex, _this.expandItem, _this.onActivateItem, onFocusItem);
            }
            switch (menuItem.itemType) {
                case 1 /* Divider */:
                    return MenuDivider(menuItem, index);
                case 2 /* Header */:
                    return MenuHeader(menuItem, index);
                default:
                    return (React.createElement(MenuItem, { key: menuItem.id, index: index, expandedIndex: _this.state.expandedIndex, expandItem: _this.expandItem, menuItem: menuItem, onActivateItem: _this.onActivateItem, onFocusItem: onFocusItem, position: position, setSize: setSize }));
            }
        };
        var empty = [];
        _this.state = {
            expandedIndex: new ObservableValue(-1),
            itemProvider: new MenuItemProvider(empty),
            items: empty
        };
        return _this;
    }
    // Produce the State from the current component props
    Menu.getDerivedStateFromProps = function (props, state) {
        var derivedItems = [];
        // Process the set of menu items.
        if (props.items) {
            var dividerItem = void 0;
            var lastItemType = 1 /* Divider */;
            for (var _i = 0, _a = props.items; _i < _a.length; _i++) {
                var menuItem = _a[_i];
                // Don't allow multiple dividers to render next to each other.
                if (menuItem.itemType === 1 /* Divider */) {
                    if (menuItem.itemType === lastItemType) {
                        continue;
                    }
                    dividerItem = menuItem;
                }
                else {
                    if (dividerItem) {
                        derivedItems.push(dividerItem);
                        dividerItem = undefined;
                    }
                    derivedItems.push(menuItem);
                }
                lastItemType = menuItem.itemType || 0 /* Normal */;
            }
        }
        return {
            itemProvider: new MenuItemProvider(derivedItems),
            items: derivedItems
        };
    };
    Menu.isDecorative = function (menuItem) {
        return menuItem.itemType === 1 /* Divider */ || menuItem.itemType === 2 /* Header */;
    };
    Menu.prototype.render = function () {
        return (React.createElement(List, { className: css(this.props.className, "bolt-menu"), columns: 7, focuszoneProps: Menu.focuszoneProps, id: this.props.id, itemProvider: this.state.itemProvider, onRenderRow: this.renderMenuItem, pageSize: Number.MAX_SAFE_INTEGER, role: "menu" }));
    };
    Menu.focuszoneProps = {
        direction: 2 /* Vertical */,
        shouldFocusOnMount: true
    };
    return Menu;
}(React.Component));
export { Menu };
export function MenuDivider(menuItem, index) {
    return (React.createElement("tr", { key: menuItem.id || "divider-" + index, className: css(menuItem.className, "bolt-menuitem-row bolt-list-row bolt-menuitem-divider"), onMouseDown: preventDefault },
        React.createElement("td", null),
        React.createElement("td", { className: "bolt-menuitem-cell bolt-list-cell bolt-menuitem-divider-column", colSpan: 5 },
            React.createElement("div", { className: "bolt-menuitem-divider-content" })),
        React.createElement("td", null)));
}
export function MenuHeader(menuItem, index) {
    return (React.createElement("tr", { key: menuItem.id || "header-" + index, className: css(menuItem.className, "bolt-menuitem-row bolt-list-row bolt-menuitem-header"), onMouseDown: preventDefault },
        React.createElement("td", { className: "bolt-menuitem-cell bolt-list-cell", colSpan: 3 }),
        React.createElement("td", { className: "bolt-menuitem-cell bolt-list-cell" },
            React.createElement("div", { className: "bolt-menuitem-cell-content bolt-menuitem-cell-text" }, menuItem.text)),
        React.createElement("td", { className: "bolt-menuitem-cell bolt-list-cell", colSpan: 3 })));
}
var MenuItemProvider = /** @class */ (function (_super) {
    tslib_1.__extends(MenuItemProvider, _super);
    function MenuItemProvider() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.positions = [];
        return _this;
    }
    // Custom getCount to support excluding the decorative items from the count.
    MenuItemProvider.prototype.getCount = function () {
        if (this.count === undefined) {
            this.count = 0;
            for (var _i = 0, _a = this.items; _i < _a.length; _i++) {
                var item = _a[_i];
                if (!Menu.isDecorative(item)) {
                    this.positions.push(++this.count);
                }
                else {
                    this.positions.push(-1);
                }
            }
        }
        return this.count;
    };
    MenuItemProvider.prototype.getPosition = function (index) {
        return this.positions[index];
    };
    return MenuItemProvider;
}(ArrayItemProvider));
var MenuItem = /** @class */ (function (_super) {
    tslib_1.__extends(MenuItem, _super);
    function MenuItem(props) {
        var _this = _super.call(this, props) || this;
        _this.localKeyStroke = false;
        _this.closeTimeout = -1;
        _this.openTimeout = -1;
        _this.handleClick = function (event) {
            var menuItem = _this.props.menuItem;
            if (menuItem.disabled || _this.state.expanded) {
                event.preventDefault();
            }
            else {
                var ownerResponse = void 0;
                // If the menu owner supplied a handler, we will get feedback from them before doing
                // default processing on the menu item.
                if (menuItem.onActivateItem) {
                    ownerResponse = menuItem.onActivateItem(menuItem, event);
                }
                // If the owner specifically returned true, we will not perform any defaults.
                if (!ownerResponse) {
                    event.preventDefault();
                    // For menus with sub-menus we will expand it on activation. For other menu items
                    // they are executed.
                    if (menuItem.subMenuProps) {
                        _this.props.expandItem(menuItem, true);
                    }
                    else if (menuItem.href) {
                        window.location.href = menuItem.href;
                    }
                    else if (menuItem.checked === undefined || menuItem.readonly) {
                        _this.props.onActivateItem(menuItem, event);
                    }
                }
            }
        };
        // If the click handler doesn't return false explicitly close dismiss the menu.
        _this.onClick = function (event) {
            if (!event.defaultPrevented) {
                _this.handleClick(event);
            }
        };
        // Called when the submenu is being dismissed.
        _this.onDismiss = function (dismissAll) {
            var menuItem = _this.props.menuItem;
            if (dismissAll) {
                /* @TODO: This seems like a problem, why are we calling onActivateItem in onDismiss, need to clean this up. */
                _this.props.onActivateItem(menuItem);
            }
            else {
                _this.props.expandItem(menuItem, false);
            }
        };
        _this.onExpandedChange = function (expandedIndex) {
            var expanded = _this.props.index === _this.props.expandedIndex.value;
            // Update the expanded state if it changed.
            if (expanded !== _this.state.expanded) {
                if (_this.element) {
                    _this.element.focus();
                }
                _this.setState({ expanded: expanded });
            }
        };
        _this.onFocus = function (event) {
            if (_this.element === document.activeElement) {
                _this.props.onFocusItem(_this.props.index, event);
            }
        };
        // Handle the keydown to expand the menu.
        _this.onKeyDown = function (event) {
            _this.localKeyStroke = true;
            if (!event.defaultPrevented) {
                var menuItem = _this.props.menuItem;
                if (event.which === 9 /* Tab */ || event.which === 32 /* Space */) {
                    event.preventDefault();
                }
                else if (event.which === 39 /* RightArrow */ && menuItem.subMenuProps) {
                    event.preventDefault();
                    _this.props.expandItem(menuItem, true);
                }
            }
        };
        // Translate the space and enter keys into onClick event for a menuItem.
        _this.onKeyUp = function (event) {
            // If we getfocus while a key is down we will get the keyup. We dont wnat
            // to process this key, it needs to originate from us.
            if (!_this.localKeyStroke) {
                return;
            }
            if (!event.defaultPrevented) {
                if (event.which === 13 /* Enter */ || event.which === 32 /* Space */) {
                    _this.handleClick(event);
                }
            }
        };
        _this.onMouseDown = function (event) {
            if (!event.defaultPrevented) {
                var menuItem = _this.props.menuItem;
                if (menuItem.disabled || _this.state.expanded) {
                    event.preventDefault();
                }
            }
        };
        // If you hover over a menu with a submenu we will open it after a short delay
        // or stop the closing timeout.
        _this.onMouseEnter = function (event) {
            var menuItem = _this.props.menuItem;
            if (menuItem.subMenuProps) {
                if (_this.openTimeout === -1) {
                    _this.openTimeout = window.setTimeout(function () {
                        _this.openTimeout = -1;
                        _this.props.expandItem(menuItem, true);
                    }, 250);
                }
                // Cancel any close that we have started.
                if (_this.closeTimeout !== -1) {
                    window.clearTimeout(_this.closeTimeout);
                    _this.closeTimeout = -1;
                }
            }
        };
        // If you leave the menu item and sub-menu we will close the menu after a short delay
        // or stop the open timeout.
        _this.onMouseLeave = function (event) {
            var menuItem = _this.props.menuItem;
            if (menuItem.subMenuProps) {
                if (_this.openTimeout !== -1) {
                    window.clearTimeout(_this.openTimeout);
                    _this.openTimeout = -1;
                }
                if (_this.state.expanded) {
                    if (_this.closeTimeout === -1) {
                        _this.closeTimeout = window.setTimeout(function () {
                            _this.closeTimeout = -1;
                            _this.props.expandItem(menuItem, false);
                        }, 250);
                    }
                }
            }
        };
        _this.state = {
            expanded: _this.props.index === _this.props.expandedIndex.value
        };
        return _this;
    }
    MenuItem.prototype.render = function () {
        var _this = this;
        var _a = this.props, menuItem = _a.menuItem, position = _a.position, setSize = _a.setSize;
        var checked = menuItem.checked, className = menuItem.className, disabled = menuItem.disabled, href = menuItem.href, iconProps = menuItem.iconProps, id = menuItem.id, onRenderMenuIcon = menuItem.onRenderMenuIcon, readonly = menuItem.readonly, secondaryText = menuItem.secondaryText, subMenuProps = menuItem.subMenuProps, text = menuItem.text;
        // If this is a link menu item we will use an anchor otherwise a plain div.
        var CellType = href ? "div" : "td";
        var RowType = href ? "a" : "tr";
        return (React.createElement(FocusZoneContext.Consumer, null, function (rowContext) { return (React.createElement(FocusWithin, { onFocus: _this.onFocus, updateStateOnFocusChange: true }, function (focusStatus) { return (React.createElement(RowType, { "aria-controls": _this.state.expanded && subMenuProps ? subMenuProps.id : undefined, "aria-disabled": disabled ? "true" : undefined, "aria-expanded": subMenuProps ? _this.state.expanded : undefined, "aria-haspopup": subMenuProps ? true : undefined, "aria-posinset": position, "aria-setsize": setSize, className: css(className, "bolt-menuitem-row bolt-list-row bolt-menuitem-row-normal", disabled && "disabled", _this.state.expanded && "expanded", focusStatus.hasFocus && "focused"), "data-focuszone": rowContext.focuszoneId, href: href, id: id, role: checked && !readonly && ObservableLike.getValue(checked) ? "menuitemcheckbox" : "menuitem", onBlur: focusStatus.onBlur, onClick: _this.onClick, onFocus: focusStatus.onFocus, onKeyDown: _this.onKeyDown, onKeyUp: _this.onKeyUp, onMouseDown: _this.onMouseDown, onMouseEnter: _this.onMouseEnter, onMouseLeave: _this.onMouseLeave, ref: function (element) { return (_this.element = element); }, tabIndex: -1 },
            React.createElement(CellType, { className: "bolt-menuitem-cell bolt-list-cell" },
                React.createElement("div", { className: "bolt-menuitem-cell-content flex-row" })),
            React.createElement(CellType, { className: "bolt-menuitem-cell bolt-list-cell" }, checked !== undefined && (React.createElement("div", { className: "bolt-menuitem-cell-content bolt-menuitem-cell-state flex-row" }, readonly === true ? (Icon({ iconName: "Checkmark" })) : (React.createElement(Checkbox, { ariaLabelledBy: id + "-text", checked: checked, disabled: disabled, excludeFocusZone: true, excludeTabStop: true, onChange: _this.onClick }))))),
            React.createElement(CellType, { className: "bolt-menuitem-cell bolt-list-cell" },
                iconProps && (React.createElement("div", { className: "bolt-menuitem-cell-content bolt-menuitem-cell-icon flex-row" }, Icon(iconProps))),
                onRenderMenuIcon && (React.createElement("div", { className: "bolt-menuitem-cell-content bolt-menuitem-cell-icon flex-row" }, onRenderMenuIcon(menuItem)))),
            React.createElement(CellType, { className: "bolt-menuitem-cell bolt-list-cell" },
                React.createElement("div", { id: id + "-text", className: "bolt-menuitem-cell-content bolt-menuitem-cell-text flex-row" },
                    React.createElement("div", { className: "text-ellipsis" }, text))),
            React.createElement(CellType, { className: "bolt-menuitem-cell bolt-list-cell" }, secondaryText && (React.createElement("div", { className: "bolt-menuitem-cell-content bolt-menuitem-cell-secondary flex-row" }, secondaryText))),
            React.createElement(CellType, { className: "bolt-menuitem-cell bolt-list-cell" }, subMenuProps && (React.createElement("div", { className: "bolt-menuitem-cell-content bolt-menuitem-cell-submenu flex-row" }, Icon({ iconName: "ChevronRight" })))),
            _this.state.expanded &&
                _this.element &&
                subMenuProps && (React.createElement(ContextualMenu, { anchorElement: _this.element, anchorOffset: { horizontal: 0, vertical: -2 }, anchorOrigin: { horizontal: "end" /* end */, vertical: "start" /* start */ }, hostSelector: ".callout-host", isSubMenu: true, menuOrigin: { horizontal: "start" /* start */, vertical: "start" /* start */ }, menuProps: subMenuProps, onDismiss: _this.onDismiss })),
            React.createElement(CellType, { className: "bolt-menuitem-cell bolt-list-cell" },
                React.createElement("div", { className: "bolt-menuitem-cell-content flex-row" })))); })); }));
    };
    MenuItem.prototype.componentDidMount = function () {
        this.props.expandedIndex.subscribe(this.onExpandedChange);
    };
    MenuItem.prototype.componentWillUnmount = function () {
        this.props.expandedIndex.unsubscribe(this.onExpandedChange);
        if (this.closeTimeout !== -1) {
            window.clearTimeout(this.closeTimeout);
        }
        if (this.openTimeout !== -1) {
            window.clearTimeout(this.openTimeout);
        }
    };
    return MenuItem;
}(React.Component));
export { MenuItem };
var ContextualMenu = /** @class */ (function (_super) {
    tslib_1.__extends(ContextualMenu, _super);
    function ContextualMenu(props) {
        var _this = _super.call(this, props) || this;
        _this.dismiss = function (dismissAll) {
            if (_this.props.onDismiss) {
                _this.props.onDismiss(dismissAll);
            }
        };
        _this.onActivateItem = function (menuItem, event) {
            if (_this.props.onDismiss) {
                _this.props.onDismiss(true);
            }
        };
        _this.onBlur = function (event) {
            if (_this.props.closeOnFocus) {
                _this.dismiss(false);
            }
        };
        _this.onIntersect = function (entries) {
            if (!_this.state.overflow) {
                for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
                    var entry = entries_1[_i];
                    if (entry.boundingClientRect.bottom > entry.rootBounds.bottom) {
                        _this.setState({ overflow: true });
                        break;
                    }
                }
            }
        };
        _this.onKeyDown = function (event) {
            if (!event.defaultPrevented) {
                if (event.which === 27 /* Escape */ || event.which === 37 /* LeftArrow */) {
                    event.preventDefault();
                    _this.dismiss(false);
                }
            }
        };
        _this.onScroll = function (event) {
            if (!event.defaultPrevented) {
                if (_this.menu) {
                    _this.menu.expandItem(null, false);
                }
            }
        };
        _this.state = {
            expandedIndex: -1,
            items: [],
            overflow: false
        };
        return _this;
    }
    ContextualMenu.prototype.render = function () {
        var _this = this;
        return (React.createElement(Callout, { anchorElement: this.props.anchorElement, anchorOffset: this.props.anchorOffset, anchorOrigin: this.props.anchorOrigin, anchorPoint: this.props.anchorPoint, calloutOrigin: this.props.menuOrigin, contentClassName: "bolt-contextual-menu", contentShadow: true, hostSelector: this.props.hostSelector, id: "bolt-contextual-menu-" + this.props.menuProps.id, isLightDismiss: true, onDismiss: this.props.onDismiss },
            React.createElement(FocusWithin, { ref: function (focus) { return (_this.focusWithin = focus); }, onBlur: this.onBlur }, function (focusStatus) { return (React.createElement(Intersection, { onIntersect: _this.onIntersect },
                React.createElement("div", { className: css("bolt-menu-container custom-scrollbar", _this.state.overflow && "overflow"), onBlur: focusStatus.onBlur, onFocus: focusStatus.onFocus, onKeyDown: _this.onKeyDown, onScroll: _this.onScroll },
                    React.createElement("div", { className: "bolt-menu-spacer", onMouseDown: preventDefault }),
                    React.createElement(Menu, tslib_1.__assign({}, _this.props.menuProps, { onActivateItem: _this.onActivateItem })),
                    React.createElement("div", { className: "bolt-menu-spacer", onMouseDown: preventDefault })))); })));
    };
    ContextualMenu.prototype.componentDidMount = function () {
        this.previousFocusElement = document.activeElement;
    };
    ContextualMenu.prototype.componentWillUnmount = function () {
        // If we still have focus as we are unmounting we will restore focus
        // to the previous focus element.
        if (this.previousFocusElement && this.focusWithin && this.focusWithin.hasFocus() && document.body.contains(this.previousFocusElement)) {
            this.previousFocusElement.focus();
        }
    };
    ContextualMenu.defaultProps = {
        closeOnFocus: true
    };
    return ContextualMenu;
}(React.Component));
export { ContextualMenu };
