import * as tslib_1 from "tslib";
import "../../FabricInit";
import "../../Core/core.scss";
import "../../Common.scss";
import "./Panel.scss";
import * as React from "react";
import { FocusTrapZone } from "office-ui-fabric-react/lib/FocusTrapZone";
import * as Resources from '../../Resources.Layer';
import { Button } from '../../Button';
import { Callout } from '../../Callout';
import { TooltipHost } from '../../Tooltip';
import { css } from '../../Util';
/**
 * A panel is a used to draw a layer that acts like a modal dialog. It captures the focus and
 * renders a view on top of the current page (generally along the left or right side.
 */
var Panel = /** @class */ (function (_super) {
    tslib_1.__extends(Panel, _super);
    function Panel(props) {
        var _this = _super.call(this, props) || this;
        _this.onDismiss = function () {
            // Call the component props dismissal function. The internal dismissal will get called when
            // the component is unmounted.
            _this.props.onDismiss && _this.props.onDismiss();
        };
        _this._panelContent = function (children) {
            return React.createElement("div", { className: "flex flex-grow vss-Panel-shown-content" }, children);
        };
        _this._panelHeader = function () {
            return (React.createElement("div", { className: "vss-Panel-header flex flex-noshrink flex-column scroll-hidden" },
                React.createElement("div", { className: "flex flex-row flex-grow scroll-hidden" },
                    React.createElement("div", { className: "title-text flex-grow title-m", tabIndex: -1 }, _this.props.title),
                    _this._panelCloseButton()),
                React.createElement("div", { className: "description-text flex-grow body-l", tabIndex: -1 }, _this.props.description)));
        };
        _this._panelBadgeHeader = function () {
            return (React.createElement("div", { className: "vss-Panel-header--badge-container flex flex-noshrink flex-column scroll-hidden" },
                React.createElement("div", { className: "vss-Panel-header--badge-title flex-row scroll-hidden" },
                    React.createElement("div", { className: "vss-Panel-header--badge-component flex-grow scroll-hidden", tabIndex: -1 }, _this.props.onRenderPanelBadgeHeader()),
                    _this._panelCloseButton()),
                React.createElement("div", { className: "vss-Panel-header--badge-content", tabIndex: -1 }, _this.props.onRenderPanelBadgeHeaderContent && _this.props.onRenderPanelBadgeHeaderContent())));
        };
        _this._panelCloseButton = function () {
            return (React.createElement(TooltipHost, { calloutProps: { beakWidth: 10 }, content: Resources.Close, directionalHint: 5 /* bottomCenter */ },
                React.createElement(Button, { className: "close-button", ariaLabel: Resources.Close, iconProps: { iconName: "ChromeClose" }, onClick: _this.props.onDismiss, subtle: true })));
        };
        _this._panelFooter = function (onRenderPanelFooter) {
            return onRenderPanelFooter && React.createElement("div", { className: "vss-Panel-footer" }, onRenderPanelFooter());
        };
        if (!props.firstFocusableSelector) {
            _this.focusElement = React.createElement("div", { className: "panelFocusElement", "aria-label": props.ariaLabel, role: "presentation", tabIndex: -1 });
        }
        return _this;
    }
    Panel.prototype.render = function () {
        return (React.createElement(Callout, { ariaLabel: this.props.ariaLabel, className: "vss-Panel", contentClassName: css("vss-Panel-content relative scroll-auto flex-grow", this.props.size === 0 /* Small */ && "size-small", this.props.size === 1 /* Medium */ && "size-medium", this.props.size === 2 /* Large */ && "size-large"), contentJustification: 3 /* Stretch */, contentLocation: 2 /* End */, contentOrientation: 0 /* Column */, contentShadow: true, id: this.props.id, isEscDismiss: true, isLightDismiss: true, isModal: false, onDismiss: this.onDismiss },
            React.createElement(FocusTrapZone, { className: "flex flex-column flex-grow scroll-hidden", firstFocusableSelector: this.props.firstFocusableSelector || "panelFocusElement", forceFocusInsideTrap: true, isClickableOutsideFocusTrap: true },
                this.focusElement,
                this.props.onRenderPanelBadgeHeader ? this._panelBadgeHeader() : this._panelHeader(),
                !!this.props.showSeparator && React.createElement("div", { className: "vss-Panel-separator flex-noshrink " }),
                this._panelContent(this.props.children),
                !!this.props.showSeparator && !!this.props.onRenderPanelFooter && React.createElement("div", { className: "vss-Panel-separator flex-noshrink " }),
                this._panelFooter(this.props.onRenderPanelFooter))));
    };
    Panel.prototype.componentWillUnmount = function () {
        // Call the layout managers dismissal function (this removes the panel from the DOM).
        this.calloutDismissed && this.calloutDismissed();
    };
    return Panel;
}(React.Component));
export { Panel };
