import * as tslib_1 from "tslib";
import * as React from "react";
import { ObservableValue } from '../../Core/Observable';
import { Checkbox } from '../../Checkbox';
import { FocusZoneContext } from '../../FocusZone';
import { Observer } from '../../Observer';
import { css, Mouse } from '../../Util';
/**
 * ColumnSelect is used to draw the selection for the given row.
 */
var ColumnSelect = /** @class */ (function () {
    function ColumnSelect() {
        var _this = this;
        this.id = "_select";
        this.width = 30;
        this.allSelected = new ObservableValue(false);
        this.initialize = function (props, table, eventDispatch) {
            _this.itemProvider = props.itemProvider;
            _this.selection = props.selection;
            // We need to know about changes to the selection to manage the selectAll state.
            if (_this.selection) {
                _this.selection.subscribe(_this.onSelectionChange);
                _this.allSelected.value = _this.itemProvider.getTotal() === _this.selection.selectedCount;
            }
        };
        this.onRenderCell = function (rowIndex, columnIndex, listColumn, listItem) {
            var selection = _this.selection;
            var selected = selection && selection.selected(rowIndex);
            return (React.createElement("td", { key: "col-select", "aria-colindex": columnIndex, className: css("bolt-table-cell bolt-list-cell", "col-" + columnIndex), "data-column-index": columnIndex },
                React.createElement("div", { className: css("bolt-table-cell-content flex-row justify-center", !selected && "bolt-table-cell-content-reveal") },
                    React.createElement("span", { className: "flex-row-inline", onMouseDown: _this.onCheckboxMouseDown },
                        React.createElement(SelectCheckbox, { checkboxProps: { checked: selected, excludeTabStop: true, excludeFocusZone: true }, rowIndex: rowIndex, selection: selection })))));
        };
        this.onRenderHeaderCell = function (columnIndex, listColumn, focuszoneId) {
            return (React.createElement(FocusZoneContext.Consumer, { key: "col-header-" + columnIndex }, function (cellContext) {
                return (React.createElement("td", { className: css("bolt-table-header-cell", "col-header-" + columnIndex), "data-column-index": columnIndex, "data-focuszone": css(columnIndex === 0 && focuszoneId, cellContext.focuszoneId), onKeyDown: _this.onKeyDown, tabIndex: columnIndex === 0 ? 0 : -1 },
                    React.createElement("div", { className: "bolt-table-header-cell-content flex-row" },
                        React.createElement(Observer, { allSelected: _this.allSelected }, function (props) {
                            var selection = _this.selection;
                            // Get the total number of items within the list.
                            var itemCount = _this.itemProvider.getTotal();
                            return (React.createElement("div", { className: css("flex-row flex-grow justify-center", !props.allSelected && "bolt-table-cell-content-reveal") }, selection &&
                                selection.multiSelect &&
                                itemCount !== -1 && (React.createElement(Checkbox, { checked: _this.allSelected, excludeFocusZone: true, excludeTabStop: true, onChange: _this.onChangeHeader }))));
                        }))));
            }));
        };
        this.onChangeHeader = function (event) {
            _this.toggleSelectAll();
        };
        this.onKeyDown = function (event) {
            if (!event.defaultPrevented) {
                if (event.which === 32 /* Space */) {
                    _this.toggleSelectAll();
                    event.preventDefault();
                }
            }
        };
        this.onCheckboxMouseDown = function (event) {
            var selection = _this.selection;
            if (selection) {
                selection.lock();
                Mouse.setCapture(function (captureEvent) {
                    if (captureEvent.type === "mouseup") {
                        // Delay the unlock until after the click has had a chance to process.
                        window.setTimeout(function () {
                            selection.unlock();
                        }, 0);
                    }
                });
            }
        };
        this.onSelectionChange = function () {
            var selection = _this.selection;
            if (selection) {
                _this.allSelected.value = _this.itemProvider.getTotal() === selection.selectedCount;
            }
        };
    }
    ColumnSelect.prototype.toggleSelectAll = function () {
        var selection = this.selection;
        if (selection) {
            if (this.allSelected.value === true) {
                selection.clear();
            }
            else {
                selection.select(0, this.itemProvider.getTotal());
            }
        }
    };
    return ColumnSelect;
}());
export { ColumnSelect };
var SelectCheckbox = /** @class */ (function (_super) {
    tslib_1.__extends(SelectCheckbox, _super);
    function SelectCheckbox() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onChange = function (event, checked) {
            var _a = _this.props, rowIndex = _a.rowIndex, selection = _a.selection;
            if (selection) {
                selection.unlock();
                if (!checked) {
                    selection.unselect(rowIndex);
                }
                else {
                    selection.select(rowIndex, 1, true);
                }
                selection.lock();
            }
        };
        return _this;
    }
    SelectCheckbox.prototype.render = function () {
        return React.createElement(Checkbox, tslib_1.__assign({}, this.props.checkboxProps, { onChange: this.onChange }));
    };
    return SelectCheckbox;
}(React.Component));
