import "../../Core/core.scss";
import "../../Common.scss";
import "./Table.scss";
import * as React from "react";
import { IEventDispatch } from '../../Utilities/Dispatch';
import { ICellPosition, ITable, ITableCell, ITableHeaderCellProps, ITableColumnGroupProps, ITableProps, ITableRowProps, ITableRowRenderer, ITableColumn } from "./Table.Props";
/**
 * ITableState is the internal state managed by the Table.
 */
export interface ITableState<T> {
    columns: Array<ITableColumn<T>>;
    eventDispatch: IEventDispatch;
    maxWidth?: string;
    minWidth?: string;
    overflow: boolean;
    tableWidth: string;
}
/**
 * ColumnFill is used to fill the remaining space in the parent element with an
 * empty column. This column can be used anywhere in the column order. Columns
 * that appear after this will be pushed to the right.
 */
export declare const ColumnFill: ITableColumn<{}>;
/**
 * The Table is a multi-row, multi-column component with an optional header.
 */
export declare class Table<T extends ITableRowRenderer<T>> extends React.Component<ITableProps<T>, ITableState<T>> implements ITable<T> {
    static getDerivedStateFromProps<T>(props: Readonly<ITableProps<T>>, state: Readonly<ITableState<T>>): Partial<ITableState<T>>;
    private list;
    constructor(props: Readonly<ITableProps<T>>);
    render(): JSX.Element;
    cellFromPoint(clientX: number, clientY: number): ICellPosition;
    getFocusIndex(): number;
    itemFromPoint(clientX: number, clientY: number): Promise<T | undefined> | T | undefined;
    indexFromPoint(clientX: number, clientY: number): number;
    private onIntersect;
    private onRenderHeader;
    private onRenderRow;
}
export interface ITableColumnGroupState<T> {
    columnWidths: string[];
}
export declare class TableColumnGroup<T> extends React.Component<ITableColumnGroupProps<T>, ITableColumnGroupState<T>> {
    static getDerivedStateFromProps<T>(props: Readonly<ITableColumnGroupProps<T>>, state: Readonly<ITableColumnGroupState<T>>): Partial<ITableColumnGroupState<T>>;
    constructor(props: Readonly<ITableColumnGroupProps<T>>);
    render(): JSX.Element;
    componentDidMount(): void;
    componentWillUnmount(): void;
    private onSizeChanged;
}
export interface ITableHeaderCellState {
    showPortal: boolean;
}
export declare class TableHeaderCell<T> extends React.Component<ITableHeaderCellProps<T>, ITableHeaderCellState> {
    private initialColumnSize;
    private initialMouseX;
    constructor(props: Readonly<ITableHeaderCellProps<T>>);
    render(): JSX.Element;
    private onMouseCapture;
    private onMouseDownSize;
}
export declare class TableRow<T> extends React.Component<ITableRowProps<T>> {
    render(): JSX.Element;
    componentDidMount(): void;
    componentWillUnmount(): void;
    private onFocus;
    private onSelectionChange;
}
export declare function renderSimpleCell(rowIndex: number, columnIndex: number, tableColumn: ITableColumn<ITableCell>, listItem: ITableCell): JSX.Element;
