import * as tslib_1 from "tslib";
import "../../Core/core.scss";
import "../../Common.scss";
import "./Table.scss";
import * as React from "react";
import { EventDispatch } from '../../Utilities/Dispatch';
import { FocusWithin } from '../../FocusWithin';
import { FocusZone, FocusZoneContext } from '../../FocusZone';
import { Intersection } from '../../Intersection';
import { List, renderListCell } from '../../List';
import { ObservableLike } from '../../Core/Observable';
import { Portal } from '../../Portal';
import { css, Mouse } from '../../Util';
import { Icon } from '../../Icon';
/**
 * ColumnFill is used to fill the remaining space in the parent element with an
 * empty column. This column can be used anywhere in the column order. Columns
 * that appear after this will be pushed to the right.
 */
export var ColumnFill = {
    id: "_fill",
    onRenderCell: function (rowIndex, columnIndex) {
        return (React.createElement("td", { key: "col-fill", "aria-colindex": columnIndex, className: css("bolt-table-cell bolt-list-cell", "col-" + columnIndex), "data-column-index": columnIndex }));
    },
    onRenderHeaderCell: function (columnIndex, tableColumn) {
        return (React.createElement("td", { key: "col-fill", className: css(tableColumn.headerClassName, "bolt-table-header-cell bolt-table-header-cell-empty", "col-header-" + columnIndex), "data-column-index": columnIndex }));
    },
    width: -100
};
/**
 * The Table is a multi-row, multi-column component with an optional header.
 */
var Table = /** @class */ (function (_super) {
    tslib_1.__extends(Table, _super);
    function Table(props) {
        var _this = _super.call(this, props) || this;
        // Reference to the underlying list interface.
        _this.list = React.createRef();
        _this.onIntersect = function (entries) {
            if (!_this.state.overflow) {
                for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
                    var entry = entries_1[_i];
                    if (entry.boundingClientRect.bottom > entry.rootBounds.bottom) {
                        _this.setState({ overflow: true });
                        break;
                    }
                }
            }
        };
        _this.onRenderHeader = function () {
            var _a = _this.props, hideHeader = _a.hideHeader, onRenderHeader = _a.onRenderHeader;
            var columns = _this.state.columns;
            return (React.createElement(React.Fragment, null,
                React.createElement(TableColumnGroup, { columns: columns }),
                !hideHeader && onRenderHeader ? onRenderHeader(columns, _this) : React.createElement(TableHeader, { tableColumns: columns, table: _this })));
        };
        _this.onRenderRow = function (index, item, position, setSize, onFocusItem) {
            // First determine if the item supplied a custom row rendering function, if not
            // attempt to use the global row rendering function.
            var onRenderRow = item.onRenderRow || _this.props.onRenderRow;
            if (onRenderRow) {
                return onRenderRow(index, item, position, setSize, _this.state.columns, onFocusItem, _this.props, _this.state.eventDispatch);
            }
            // If no custom row renderer is available use the default row renderer.
            return (React.createElement(TableRow, { key: index, columns: _this.state.columns, tableProps: _this.props, eventDispatch: _this.state.eventDispatch, index: index, item: item, onFocusItem: onFocusItem, position: position, setSize: setSize }));
        };
        _this.state = {
            columns: [],
            eventDispatch: props.eventDispatch || new EventDispatch(),
            overflow: false,
            tableWidth: ""
        };
        // Initialize any columns that requested initialization.
        for (var _i = 0, _a = props.columns; _i < _a.length; _i++) {
            var tableColumn = _a[_i];
            if (tableColumn.initialize) {
                tableColumn.initialize(props, _this, _this.state.eventDispatch);
            }
        }
        // Initialize the supplied behaviors.
        if (props.behaviors) {
            for (var _b = 0, _c = props.behaviors; _b < _c.length; _b++) {
                var behavior = _c[_b];
                behavior.initialize(props, _this, _this.state.eventDispatch);
            }
        }
        return _this;
    }
    Table.getDerivedStateFromProps = function (props, state) {
        var columns = [];
        var tableMaxWidth = 0;
        var tableMinWidth = 0;
        var tableWidth = 0;
        var constrain = false;
        var fill = false;
        for (var _i = 0, _a = props.columns; _i < _a.length; _i++) {
            var column = _a[_i];
            var maxWidth = column.maxWidth, minWidth = column.minWidth;
            var width = ObservableLike.getValue(column.width);
            if (width < 0) {
                // This is a variable width columnn so we will fill the container.
                fill = true;
                // Update the min/max width of the table based on the supplied value.
                // We use a minumum width of 100px if one isnt specified.
                tableMinWidth += minWidth ? minWidth : 0;
                // If we have a variable width column, we will constrain the width by the sum
                // of the fixed width columns and the maxWidth of the variable width columns.
                if (maxWidth) {
                    constrain = true;
                    tableMaxWidth += maxWidth;
                }
            }
            else {
                tableWidth += width;
                tableMinWidth += width;
                tableMaxWidth += width;
            }
            columns.push(column);
        }
        return {
            columns: columns,
            maxWidth: constrain ? tableMaxWidth + "px" : undefined,
            minWidth: tableMinWidth !== tableWidth ? tableMinWidth + "px" : undefined,
            tableWidth: fill ? "100%" : tableWidth + "px"
        };
    };
    Table.prototype.render = function () {
        var _a = this.props, className = _a.className, focuszoneProps = _a.focuszoneProps, hideLines = _a.hideLines, id = _a.id, itemProvider = _a.itemProvider, onActivate = _a.onActivate, onSelect = _a.onSelect, rowHeight = _a.rowHeight, scrollable = _a.scrollable, selection = _a.selection;
        var _b = this.state, columns = _b.columns, eventDispatch = _b.eventDispatch, maxWidth = _b.maxWidth, minWidth = _b.minWidth, overflow = _b.overflow, tableWidth = _b.tableWidth;
        var listProps = {
            className: css(className, "bolt-table", !hideLines && "bolt-table-show-lines"),
            columns: columns.length,
            eventDispatch: eventDispatch,
            focuszoneProps: focuszoneProps,
            id: id,
            itemProvider: itemProvider,
            maxWidth: maxWidth,
            minWidth: minWidth,
            onActivate: onActivate,
            onRenderHeader: this.onRenderHeader,
            onRenderRow: this.onRenderRow,
            onSelect: onSelect,
            role: "grid",
            rowHeight: rowHeight,
            selection: selection,
            width: tableWidth
        };
        var table = (React.createElement("div", { className: css("bolt-table-container relative", scrollable && "scroll-auto custom-scrollbar") },
            React.createElement(List, tslib_1.__assign({}, listProps, { ref: this.list }))));
        if (scrollable) {
            table = (React.createElement(Intersection, { rootMargin: window.innerHeight / 2 + "px", onIntersect: this.onIntersect }, table));
        }
        return table;
    };
    Table.prototype.cellFromPoint = function (clientX, clientY) {
        var attributeValue;
        var columnIndex = -1;
        var rowIndex = -1;
        var element = document.elementFromPoint(clientX, clientY);
        while (element) {
            attributeValue = element.getAttribute("data-column-index");
            if (attributeValue) {
                columnIndex = parseInt(attributeValue, 10);
            }
            attributeValue = element.getAttribute("data-row-index");
            if (attributeValue) {
                rowIndex = parseInt(attributeValue, 10);
                break;
            }
            // We have hit the root of the details list, dont look above this.
            if (element.classList.contains("bolt-table")) {
                break;
            }
            element = element.parentElement;
        }
        return { columnIndex: columnIndex, rowIndex: rowIndex };
    };
    Table.prototype.getFocusIndex = function () {
        if (this.list.current) {
            return this.list.current.getFocusIndex();
        }
        return -1;
    };
    Table.prototype.itemFromPoint = function (clientX, clientY) {
        var cellPosition = this.cellFromPoint(clientX, clientY);
        if (cellPosition.rowIndex >= 0) {
            return this.props.itemProvider.getItem(cellPosition.rowIndex);
        }
        return undefined;
    };
    Table.prototype.indexFromPoint = function (clientX, clientY) {
        if (this.list.current) {
            return this.list.current.indexFromPoint(clientX, clientY);
        }
        return -1;
    };
    return Table;
}(React.Component));
export { Table };
var TableHeader = /** @class */ (function (_super) {
    tslib_1.__extends(TableHeader, _super);
    function TableHeader() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onKeyDown = function (event) {
            if (!event.defaultPrevented) {
                if (event.which === 13 /* Enter */ || event.which === 32 /* Space */) {
                    event.preventDefault();
                }
            }
        };
        return _this;
    }
    TableHeader.prototype.render = function () {
        var _this = this;
        return (React.createElement(FocusZoneContext.Consumer, null, function (rowContext) {
            return (React.createElement(FocusZone, { direction: 1 /* Horizontal */ },
                React.createElement("thead", null,
                    React.createElement(FocusWithin, { updateStateOnFocusChange: true }, function (focusStatus) {
                        return (React.createElement("tr", { className: css("bolt-table-header-row", focusStatus.hasFocus && "focused"), "data-row-index": -1, onBlur: focusStatus.onBlur, onFocus: focusStatus.onFocus, onKeyDown: _this.onKeyDown }, _this.props.tableColumns.map(function (column, columnIndex) {
                            if (column.onRenderHeaderCell) {
                                return column.onRenderHeaderCell(columnIndex, column, rowContext.focuszoneId);
                            }
                            else {
                                return (React.createElement(TableHeaderCell, { key: "col-header-" + columnIndex, column: column, columnIndex: columnIndex, focuszoneId: rowContext.focuszoneId },
                                    React.createElement("div", { className: "bolt-table-header-cell-text" },
                                        column.iconProps && Icon(column.iconProps),
                                        column.name)));
                            }
                        })));
                    }),
                    React.createElement("tr", { className: "bolt-table-header-row", "data-row-index": -1 }, _this.props.tableColumns.map(function (column, columnIndex) {
                        return (React.createElement("td", { key: "col-header-" + columnIndex, className: css("bolt-table-header-spacer", "col-header-" + columnIndex), "data-column-index": columnIndex }));
                    })))));
        }));
    };
    return TableHeader;
}(React.Component));
var TableColumnGroup = /** @class */ (function (_super) {
    tslib_1.__extends(TableColumnGroup, _super);
    function TableColumnGroup(props) {
        var _this = _super.call(this, props) || this;
        _this.onSizeChanged = function () {
            _this.forceUpdate();
        };
        _this.state = { columnWidths: [] };
        return _this;
    }
    TableColumnGroup.getDerivedStateFromProps = function (props, state) {
        var columnWidths = [];
        for (var _i = 0, _a = props.columns; _i < _a.length; _i++) {
            var column = _a[_i];
            var width = ObservableLike.getValue(column.width);
            if (width < 0) {
                columnWidths.push(Math.abs(width) + "%");
            }
            else {
                columnWidths.push(width + "px");
            }
        }
        return { columnWidths: columnWidths };
    };
    TableColumnGroup.prototype.render = function () {
        var _this = this;
        return (React.createElement("colgroup", null, this.props.columns.map(function (column, columnIndex) {
            return React.createElement("col", { key: "col-header-" + columnIndex, style: { width: _this.state.columnWidths[columnIndex] } });
        })));
    };
    TableColumnGroup.prototype.componentDidMount = function () {
        for (var _i = 0, _a = this.props.columns; _i < _a.length; _i++) {
            var column = _a[_i];
            if (ObservableLike.isObservable(column.width)) {
                column.width.subscribe(this.onSizeChanged);
            }
        }
    };
    TableColumnGroup.prototype.componentWillUnmount = function () {
        for (var _i = 0, _a = this.props.columns; _i < _a.length; _i++) {
            var column = _a[_i];
            if (ObservableLike.isObservable(column.width)) {
                column.width.unsubscribe(this.onSizeChanged);
            }
        }
    };
    return TableColumnGroup;
}(React.Component));
export { TableColumnGroup };
var TableHeaderCell = /** @class */ (function (_super) {
    tslib_1.__extends(TableHeaderCell, _super);
    function TableHeaderCell(props) {
        var _this = _super.call(this, props) || this;
        _this.initialColumnSize = 0;
        _this.initialMouseX = 0;
        _this.onMouseCapture = function (event) {
            var column = _this.props.column;
            var sizeChange = event.clientX - _this.initialMouseX;
            var columnSize = Math.floor(Math.min(column.maxWidth || 100000, Math.max(column.minWidth || 100, _this.initialColumnSize + sizeChange)));
            // Remove the portal now that we are no longer sizing.
            if (event.type === "mouseup") {
                _this.setState({ showPortal: false });
            }
            column.onSize(_this.props.columnIndex, columnSize);
        };
        _this.onMouseDownSize = function (event) {
            _this.initialColumnSize = ObservableLike.getValue(_this.props.column.width);
            _this.initialMouseX = event.clientX;
            // Capture the mouse, this will let us size the column even when the mouse moves
            // outside our element.
            Mouse.setCapture(_this.onMouseCapture);
            // Show the portal that keeps our sizing actions for effecting the other elements.
            _this.setState({ showPortal: true });
            // Don't let the event start a mouse selection or perform some other task.
            event.preventDefault();
        };
        _this.state = { showPortal: false };
        return _this;
    }
    TableHeaderCell.prototype.render = function () {
        var _this = this;
        var _a = this.props, column = _a.column, columnIndex = _a.columnIndex, focuszoneId = _a.focuszoneId;
        return (React.createElement(FocusZoneContext.Consumer, null, function (cellContext) {
            return (React.createElement("td", { className: css(column.headerClassName, "bolt-table-header-cell", "col-header-" + columnIndex), "data-column-index": columnIndex, "data-focuszone": css(columnIndex === 0 && focuszoneId, cellContext.focuszoneId), tabIndex: columnIndex === 0 ? 0 : -1 },
                React.createElement("div", { className: "bolt-table-header-cell-content flex-row" },
                    column.onSize && React.createElement("div", { className: "bolt-table-header-sizer", onMouseDown: _this.onMouseDownSize }),
                    _this.props.children),
                _this.state.showPortal && (React.createElement(Portal, { className: "bolt-table-size-portal" },
                    React.createElement("div", null)))));
        }));
    };
    return TableHeaderCell;
}(React.Component));
export { TableHeaderCell };
var TableRow = /** @class */ (function (_super) {
    tslib_1.__extends(TableRow, _super);
    function TableRow() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onFocus = function (event) {
            _this.props.onFocusItem(_this.props.index, event);
        };
        _this.onSelectionChange = function (selectedRanges) {
            var index = _this.props.index;
            for (var _i = 0, selectedRanges_1 = selectedRanges; _i < selectedRanges_1.length; _i++) {
                var selectionRange = selectedRanges_1[_i];
                if (index >= selectionRange.beginIndex && index <= selectionRange.endIndex) {
                    _this.forceUpdate();
                }
            }
        };
        return _this;
    }
    TableRow.prototype.render = function () {
        var _this = this;
        var _a = this.props, columns = _a.columns, tableProps = _a.tableProps, index = _a.index, item = _a.item, position = _a.position, setSize = _a.setSize;
        var selection = tableProps.selection;
        var selected = selection && selection.selected(index);
        return (React.createElement(FocusZoneContext.Consumer, null, function (rowContext) {
            return (React.createElement(FocusWithin, { onFocus: _this.onFocus, updateStateOnFocusChange: true }, function (focusStatus) {
                return (React.createElement(FocusZone, { direction: 1 /* Horizontal */ },
                    React.createElement("tr", { "aria-posinset": position, "aria-rowindex": _this.props.index + 1, "aria-setsize": setSize, className: css("bolt-table-row bolt-list-row", focusStatus.hasFocus && "focused", selected && "selected"), "data-focuszone": rowContext.focuszoneId, "data-row-index": _this.props.index, tabIndex: -1, onBlur: focusStatus.onBlur, onFocus: focusStatus.onFocus }, columns.map(function (tableColumn, columnIndex) {
                        return tableColumn.onRenderCell(index, columnIndex, tableColumn, item);
                    }))));
            }));
        }));
    };
    TableRow.prototype.componentDidMount = function () {
        var selection = this.props.tableProps.selection;
        if (selection) {
            selection.subscribe(this.onSelectionChange);
        }
    };
    TableRow.prototype.componentWillUnmount = function () {
        var selection = this.props.tableProps.selection;
        if (selection) {
            selection.unsubscribe(this.onSelectionChange);
        }
    };
    return TableRow;
}(React.Component));
export { TableRow };
export function renderSimpleCell(rowIndex, columnIndex, tableColumn, listItem) {
    var className = tableColumn.className, columnStyle = tableColumn.columnStyle, disabled = tableColumn.disabled;
    var listCell = listItem[tableColumn.id];
    return (React.createElement("td", { key: "col-" + columnIndex, "aria-colindex": columnIndex, className: css(className, "bolt-table-cell bolt-list-cell", "col-" + columnIndex, disabled && "disabled", columnStyle === 1 /* Primary */ && "bolt-table-cell-primary", columnStyle === 0 /* Secondary */ && "bolt-table-cell-secondary", columnStyle === 2 /* Tertiary */ && "bolt-table-cell-tertiary"), "data-column-index": columnIndex },
        React.createElement("div", { className: "bolt-table-cell-content bolt-list-cell-content flex-row" }, listCell && renderListCell(listCell))));
}
