import * as tslib_1 from "tslib";
import * as React from "react";
import { Callout } from '../../Callout';
import { css, Mouse } from '../../Util';
var Tooltip = /** @class */ (function (_super) {
    tslib_1.__extends(Tooltip, _super);
    function Tooltip(props) {
        var _this = _super.call(this, props) || this;
        _this.openTimeout = -1;
        _this.state = {
            visible: false
        };
        return _this;
    }
    Tooltip.prototype.render = function () {
        var _this = this;
        var child = React.Children.only(this.props.children);
        // Save the existing mouse functions and call them before processing the events.
        var existingMouseEnter = child.props.onMouseEnter;
        var existingMouseLeave = child.props.onMouseLeave;
        var clonedChild = React.cloneElement(child, Object.assign({}, child.props, {
            onMouseEnter: function (event) {
                if (!_this.props.disabled) {
                    var anchorElement_1 = event.currentTarget;
                    // If the tooltip only appears when the anchorElement overflows its parent then
                    // we need to check on mouse enter.
                    if (_this.props.overflowOnly) {
                        if (anchorElement_1.parentElement &&
                            anchorElement_1.scrollWidth <= Math.ceil(anchorElement_1.parentElement.getBoundingClientRect().width)) {
                            return;
                        }
                    }
                    if (_this.openTimeout === -1) {
                        _this.openTimeout = window.setTimeout(function () {
                            _this.openTimeout = -1;
                            // If no anchorOrigin was specified use the Mouse.position when we show the toolip.
                            var anchorPoint;
                            if (!_this.props.anchorOrigin) {
                                anchorPoint = Mouse.position;
                            }
                            _this.setState({
                                anchorElement: anchorElement_1,
                                anchorPoint: anchorPoint,
                                innerText: _this.props.overflowOnly && !_this.props.text && anchorElement_1.innerText,
                                visible: true
                            });
                        }, _this.props.delayMs);
                    }
                }
                if (existingMouseEnter) {
                    existingMouseEnter(event);
                }
            },
            onMouseLeave: function (event) {
                if (_this.openTimeout !== -1) {
                    window.clearTimeout(_this.openTimeout);
                    _this.openTimeout = -1;
                }
                if (_this.state.visible) {
                    _this.setState({ anchorElement: undefined, visible: false });
                }
                if (existingMouseLeave) {
                    existingMouseLeave(event);
                }
            }
        }), child.props.children);
        return (React.createElement(React.Fragment, null,
            clonedChild,
            this.state.visible && !this.props.disabled && this.state.anchorElement ? (React.createElement(Callout, { anchorElement: this.state.anchorElement, anchorOffset: this.props.anchorOffset || { horizontal: 8, vertical: 8 }, anchorOrigin: this.props.anchorOrigin || { horizontal: "center" /* center */, vertical: "end" /* end */ }, anchorPoint: this.state.anchorPoint, calloutOrigin: this.props.tooltipOrigin, className: css(this.props.className, "bolt-tooltip"), hostElement: this.props.hostElement, hostSelector: this.props.hostSelector },
                React.createElement("div", { className: "bolt-tooltip-content" }, (this.props.onRenderContent && this.props.onRenderContent()) || this.props.text || this.state.innerText))) : null));
    };
    Tooltip.prototype.componentWillUnmount = function () {
        if (this.openTimeout !== -1) {
            window.clearTimeout(this.openTimeout);
        }
    };
    Tooltip.defaultProps = {
        delayMs: 250
    };
    return Tooltip;
}(React.Component));
export { Tooltip };
