export declare const enum Location {
    start = "start",
    center = "center",
    end = "end"
}
export interface IOrigin {
    horizontal: Location;
    vertical: Location;
}
export interface IOffset {
    horizontal: number;
    vertical: number;
}
export interface IPoint {
    x: number;
    y: number;
}
/**
 * The position method is used to set the location of an absolutely positioned element
 * using the standard positioning properties. The names of these properties conform to
 * the naming patterns used in the Material Popover https://material-ui.com. They
 * are not exact but follow the same pattern.
 *
 * For an example usage, look at the Callout component and how it uses this method to
 * position the element in the page.
 *
 * @param transformElement The element that is being positioned/transformed.
 * @param transformOrigin The origin within the transformed element to align with the
 *  anchor position.
 * @param anchorOffset Offset on the anchorElement that is applied to the computed location
 *  given the element/origin/point.
 * @param anchorElement The element used to anchor the position of the transformed element.
 *  The caller must supply either an anchorElement and anchorOrigin, or anchorPoint.
 * @param anchorOrigin When an anchorElement is supplie the anchorOrigin defines the location
 *  on the anchorElement used for positioning.
 * @param anchorPoint Instead of an anchorElement the caller can use an explicit point
 *  to be used as the basis for the anchorLocation. The anchorOffset will still be applied.
 * @param pageOffset If the caller is changing the DOM and doesnt want a force reflow,
 *  they may pass the current pageOffset to be used. Note that if the page changes/scrolls
 *  while the element is being laid out the positioning may not be accurate.
 */
export declare function position(transformElement: HTMLElement, transformOrigin: IOrigin, anchorOffset?: IOffset, anchorElement?: HTMLElement, anchorOrigin?: IOrigin, anchorPoint?: IPoint, pageOffset?: IPoint): void;
