import * as tslib_1 from "tslib";
import { Observable } from '../Core/Observable';
var Selection = /** @class */ (function (_super) {
    tslib_1.__extends(Selection, _super);
    function Selection(multiSelect) {
        var _this = _super.call(this) || this;
        _this.selectedCount = 0;
        _this.selectedRanges = [];
        _this.lockCount = 0;
        _this.multiSelect = multiSelect || false;
        return _this;
    }
    Selection.prototype.clear = function () {
        if (!this.lockCount) {
            // Save the current selection ranges for notification.
            var selectedRanges = this.selectedRanges;
            // Reset the selection to an empty selection.
            this.selectedRanges = [];
            this.selectedCount = 0;
            // Go through and notify any observers of the change.
            this.notify(selectedRanges, "unselect");
        }
    };
    Selection.prototype.getSelection = function () {
        return this.selectedRanges;
    };
    Selection.prototype.selected = function (index) {
        for (var _i = 0, _a = this.selectedRanges; _i < _a.length; _i++) {
            var selectionRange = _a[_i];
            if (index >= selectionRange.beginIndex && index <= selectionRange.endIndex) {
                return true;
            }
        }
        return false;
    };
    Selection.prototype.select = function (index, count, merge) {
        if (!this.lockCount) {
            var updatedRange = void 0;
            // If no count is specified we will use a single item selection.
            count = count || 1;
            if (!this.multiSelect) {
                if (!this.selected(index)) {
                    this.clear();
                    updatedRange = { beginIndex: index, endIndex: index };
                    this.selectedRanges.push(updatedRange);
                }
            }
            else {
                if (!merge) {
                    this.clear();
                }
                // @TODO: Implement a more optimal multi-count selection
                for (; count > 0; count--) {
                    var rangeIndex = 0;
                    // Determine if there is a range we can add this selection too.
                    for (; rangeIndex < this.selectedRanges.length; rangeIndex++) {
                        var selectionRange = this.selectedRanges[rangeIndex];
                        // Check if this selection occurs before this selectionRange.
                        if (index < selectionRange.beginIndex) {
                            if (index === selectionRange.beginIndex - 1) {
                                updatedRange = selectionRange;
                                updatedRange.beginIndex--;
                            }
                            break;
                        }
                        // If this index is directly after this range we will extend it.
                        else if (index === selectionRange.endIndex + 1) {
                            // If there is a gap of 1 number we will merge the two ranges.
                            if (rangeIndex < this.selectedRanges.length - 1 && index === this.selectedRanges[rangeIndex + 1].beginIndex - 1) {
                                updatedRange = selectionRange;
                                updatedRange.endIndex = this.selectedRanges[rangeIndex + 1].endIndex;
                                // Remove the second range since it is merged into the previous range.
                                this.selectedRanges.splice(rangeIndex + 1, 1);
                            }
                            else {
                                updatedRange = selectionRange;
                                updatedRange.endIndex++;
                            }
                            break;
                        }
                    }
                    // If there was no range to merge with, add a new one.
                    if (!updatedRange) {
                        updatedRange = { beginIndex: index, endIndex: index };
                        this.selectedRanges.splice(rangeIndex, 0, updatedRange);
                    }
                    this.selectedCount++;
                    index++;
                }
            }
            // Notify observers of the added selection.
            if (updatedRange) {
                this.notify([updatedRange], "select");
            }
        }
    };
    Selection.prototype.unselect = function (index, count) {
        if (!this.lockCount) {
            var updatedRange = void 0;
            // If no count is specified we will use a single item selection.
            count = count || 1;
            // @TODO: Implement a more optimal multi-count selection
            for (; count > 0; count--) {
                // Determine if there is a range we can add this selection too.
                for (var rangeIndex = 0; rangeIndex < this.selectedRanges.length; rangeIndex++) {
                    var selectionRange = this.selectedRanges[rangeIndex];
                    // If this index if before this range move on to the next one.
                    if (index < selectionRange.beginIndex) {
                        continue;
                    }
                    // Determine whether or not this index falls into this range.
                    if (index >= selectionRange.beginIndex && index <= selectionRange.endIndex) {
                        // If the index is on the start or end of the range, we will just shrink it.
                        // Otherwise we will have to split it.
                        if (index === selectionRange.beginIndex) {
                            selectionRange.beginIndex++;
                        }
                        else if (index === selectionRange.endIndex) {
                            selectionRange.endIndex--;
                        }
                        else if (index > selectionRange.beginIndex && index < selectionRange.endIndex) {
                            this.selectedRanges.splice(rangeIndex, 0, {
                                beginIndex: index + 1,
                                endIndex: selectionRange.endIndex
                            });
                            selectionRange.endIndex = index - 1;
                        }
                        // Shrinking may have created an empty range, we need to remove it.
                        if (selectionRange.beginIndex > selectionRange.endIndex) {
                            this.selectedRanges.splice(rangeIndex, 1);
                        }
                        // Create a range we use to notify the observers.
                        updatedRange = { beginIndex: index, endIndex: index };
                        break;
                    }
                }
                this.selectedCount--;
            }
            if (updatedRange) {
                this.notify([updatedRange], "unselect");
            }
        }
    };
    Selection.prototype.lock = function () {
        this.lockCount++;
    };
    Selection.prototype.unlock = function () {
        this.lockCount--;
    };
    return Selection;
}(Observable));
export { Selection };
